# -*- coding: utf-8 -*-
import copy
import json
import os
import re
import shutil
import subprocess
import time

from requests_toolbelt import MultipartEncoder

from . import config


def download_video(self, media_id, filename=None, media=False, folder="videos"):
    video_urls = []
    if not media:
        self.media_info(media_id)
        media = self.last_json["items"][0]
    filename = (
        "{}_{}.mp4".format(media["user"]["username"], media_id)
        if not filename
        else "{}.mp4".format(filename)
    )

    try:
        clips = media["video_versions"]
        video_urls.append(clips[0]["url"])
    except KeyError:
        carousels = media.get("carousel_media", [])
        for carousel in carousels:
            video_urls.append(carousel["video_versions"][0]["url"])
    except Exception:
        return False

    fname = os.path.join(folder, filename)
    if os.path.exists(fname):
        return os.path.abspath(fname)

    for counter, video_url in enumerate(video_urls):
        response = self.session.get(video_url, stream=True)
        if response.status_code == 200:
            fname = os.path.join(folder, "{}_{}".format(counter, filename))
            with open(fname, "wb") as f:
                response.raw.decode_content = True
                shutil.copyfileobj(response.raw, f)

    return os.path.abspath(fname)


# leaving here function used by old upload_video, no more used now
def get_video_info(filename):
    res = {}
    try:
        terminalResult = subprocess.Popen(
            ["ffprobe", filename], stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        for x in terminalResult.stdout.readlines():
            # Duration: 00:00:59.51, start: 0.000000, bitrate: 435 kb/s
            m = re.search(
                r"duration: (\d\d:\d\d:\d\d\.\d\d),", str(x), flags=re.IGNORECASE
            )
            if m is not None:
                res["duration"] = m.group(1)
            # Video: h264 (Constrained Baseline) (avc1 / 0x31637661), yuv420p, 480x268
            m = re.search(r"video:\s.*\s(\d+)x(\d+)\s", str(x), flags=re.IGNORECASE)
            if m is not None:
                res["width"] = m.group(1)
                res["height"] = m.group(2)
    finally:
        if "width" not in res:
            print(
                "ERROR: 'ffprobe' not found, please install "
                "'ffprobe' with one of following methods:"
            )
            print("   sudo apt-get install ffmpeg")
            print("or sudo apt-get install -y libav-tools")
    return res


def upload_video(self, video, caption=None, upload_id=None, thumbnail=None, options={}):
    """Upload video to Instagram

    @param video      Path to video file (String)
    @param caption    Media description (String)
    @param upload_id  Unique upload_id (String). When None, then generate automatically
    @param thumbnail  Path to thumbnail for video (String). When None, then thumbnail is generate automatically
    @param options    Object with difference options, e.g. configure_timeout, rename_thumbnail, rename (Dict)
                      Designed to reduce the number of function arguments!
                      This is the simplest request object.

    @return           Object with state of uploading to Instagram (or False)
    """
    options = dict(
        {"configure_timeout": 15, "rename_thumbnail": True, "rename": True},
        **(options or {})
    )
    if upload_id is None:
        upload_id = str(int(time.time() * 1000))
    video, thumbnail, width, height, duration = resize_video(video, thumbnail)
    data = {
        "upload_id": upload_id,
        "_csrftoken": self.token,
        "media_type": "2",
        "_uuid": self.uuid,
    }
    m = MultipartEncoder(data, boundary=self.uuid)
    self.session.headers.update(
        {
            "X-IG-Capabilities": "3Q4=",
            "X-IG-Connection-Type": "WIFI",
            "Host": "i.instagram.com",
            "Cookie2": "$Version=1",
            "Accept-Language": "en-US",
            "Accept-Encoding": "gzip, deflate",
            "Content-type": m.content_type,
            "Connection": "keep-alive",
            "User-Agent": self.user_agent,
        }
    )
    response = self.session.post(config.API_URL + "upload/video/", data=m.to_string())
    if response.status_code == 200:
        body = json.loads(response.text)
        upload_url = body["video_upload_urls"][3]["url"]
        upload_job = body["video_upload_urls"][3]["job"]

        with open(video, "rb") as video_bytes:
            video_data = video_bytes.read()
        # solve issue #85 TypeError: slice indices must be integers or None or have an __index__ method
        request_size = len(video_data) // 4
        last_request_extra = len(video_data) - 3 * request_size

        headers = copy.deepcopy(self.session.headers)
        self.session.headers.update(
            {
                "X-IG-Capabilities": "3Q4=",
                "X-IG-Connection-Type": "WIFI",
                "Cookie2": "$Version=1",
                "Accept-Language": "en-US",
                "Accept-Encoding": "gzip, deflate",
                "Content-type": "application/octet-stream",
                "Session-ID": upload_id,
                "Connection": "keep-alive",
                "Content-Disposition": 'attachment; filename="video.mov"',
                "job": upload_job,
                "Host": "upload.instagram.com",
                "User-Agent": self.user_agent,
            }
        )
        for i in range(4):
            start = i * request_size
            if i == 3:
                end = i * request_size + last_request_extra
            else:
                end = (i + 1) * request_size
            length = last_request_extra if i == 3 else request_size
            content_range = "bytes {start}-{end}/{len_video}".format(
                start=start, end=end - 1, len_video=len(video_data)
            ).encode("utf-8")

            self.session.headers.update(
                {"Content-Length": str(end - start), "Content-Range": content_range}
            )
            response = self.session.post(
                upload_url, data=video_data[start: start + length]
            )
        self.session.headers = headers

        configure_timeout = options.get("configure_timeout")
        if response.status_code == 200:
            for attempt in range(4):
                if configure_timeout:
                    time.sleep(configure_timeout)
                if self.configure_video(
                    upload_id,
                    video,
                    thumbnail,
                    width,
                    height,
                    duration,
                    caption,
                    options=options,
                ):
                    media = self.last_json.get("media")
                    self.expose()
                    if options.get("rename"):
                        from os import rename

                        rename(video, "{}.REMOVE_ME".format(video))
                    return media
    return False


def configure_video(
    self, upload_id, video, thumbnail, width, height, duration, caption="", options={}
):
    """Post Configure Video (send caption, thumbnail and more else to Instagram)

    @param upload_id  Unique upload_id (String). Received from "upload_video"
    @param video      Path to video file (String)
    @param thumbnail  Path to thumbnail for video (String). When None, then thumbnail is generate automatically
    @param width      Width in px (Integer)
    @param height     Height in px (Integer)
    @param duration   Duration in seconds (Integer)
    @param caption    Media description (String)
    @param options    Object with difference options, e.g. configure_timeout, rename_thumbnail, rename (Dict)
                      Designed to reduce the number of function arguments!
                      This is the simplest request object.
    """
    # clipInfo = get_video_info(video)
    options = {"rename": options.get("rename_thumbnail", True)}
    self.upload_photo(
        photo=thumbnail,
        caption=caption,
        upload_id=upload_id,
        from_video=True,
        options=options,
    )
    data = self.json_data(
        {
            "upload_id": upload_id,
            "source_type": 3,
            "poster_frame_index": 0,
            "length": 0.00,
            "audio_muted": False,
            "filter_type": 0,
            "video_result": "deprecated",
            "clips": {
                "length": duration,
                "source_type": "3",
                "camera_position": "back",
            },
            "extra": {"source_width": width, "source_height": height},
            "device": self.device_settings,
            "caption": caption,
        }
    )
    return self.send_request("media/configure/?video=1", data)


def resize_video(fname, thumbnail=None):
    from math import ceil

    try:
        import moviepy.editor as mp
    except ImportError as e:
        print("ERROR: {}".format(e))
        print(
            "Required module `moviepy` not installed\n"
            "Install with `pip install moviepy` and retry.\n\n"
            "You may need also:\n"
            "pip install --upgrade setuptools\n"
            "pip install numpy --upgrade --ignore-installed"
        )
        return False
    print("Analizing `{}`".format(fname))
    h_lim = {"w": 90.0, "h": 47.0}
    v_lim = {"w": 4.0, "h": 5.0}
    d_lim = 60
    vid = mp.VideoFileClip(fname)
    (w, h) = vid.size
    deg = vid.rotation
    ratio = w * 1.0 / h * 1.0
    print(
        "FOUND w:{w}, h:{h}, rotation={d}, ratio={r}".format(w=w, h=h, r=ratio, d=deg)
    )
    if w > h:
        print("Horizontal video")
        if ratio > (h_lim["w"] / h_lim["h"]):
            print("Cropping video")
            cut = int(ceil((w - h * h_lim["w"] / h_lim["h"]) / 2))
            left = cut
            right = w - cut
            top = 0
            bottom = h
            vid = vid.crop(x1=left, y1=top, x2=right, y2=bottom)
            (w, h) = vid.size
        if w > 1080:
            print("Resizing video")
            vid = vid.resize(width=1080)
    elif w < h:
        print("Vertical video")
        if ratio < (v_lim["w"] / v_lim["h"]):
            print("Cropping video")
            cut = int(ceil((h - w * v_lim["h"] / v_lim["w"]) / 2))
            left = 0
            right = w
            top = cut
            bottom = h - cut
            vid = vid.crop(x1=left, y1=top, x2=right, y2=bottom)
            (w, h) = vid.size
        if h > 1080:
            print("Resizing video")
            vid = vid.resize(height=1080)
    else:
        print("Square video")
        if w > 1080:
            print("Resizing video")
            vid = vid.resize(width=1080)
    (w, h) = vid.size
    if vid.duration > d_lim:
        print("Cutting video to {} sec from start".format(d_lim))
        vid = vid.subclip(0, d_lim)
    new_fname = "{}.CONVERTED.mp4".format(fname)
    print("Saving new video w:{w} h:{h} to `{f}`".format(w=w, h=h, f=new_fname))
    vid.write_videofile(new_fname, codec="libx264", audio_codec="aac")
    if not thumbnail:
        print("Generating thumbnail...")
        thumbnail = "{}.jpg".format(fname)
        vid.save_frame(thumbnail, t=(vid.duration / 2))
    return new_fname, thumbnail, w, h, vid.duration



DEFAULT_RESPONSE = {"status": "ok"}

TEST_CAPTION_ITEM = {
    "bit_flags": 0,
    "content_type": "comment",
    "created_at": 1494733796,
    "created_at_utc": 1494733796,
    "did_report_as_spam": False,
    "pk": 17856098620165444,
    "status": "Active",
    "text": "Old Harry Rocks, Dorset, UK\n\n#oldharryrocks #dorset #uk #rocks #clouds #water #photoshoot #nature #amazing #beautifulsky #sky #landscape #nice #beautiful #awesome #landscapes #l4l #f4f",
    "type": 1,
    "user": {
        "full_name": "The best earth places",
        "is_private": False,
        "is_verified": False,
        "pk": 182696006,
        "profile_pic_id": "1477989239094674784_182696006",
        "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/703a0877fc8653d8b1ec17d57cf0aed6/5B2669EF/t51.2885-19/s150x150/17332952_229750480834181_3303899473574363136_a.jpg",
        "username": "best.earth.places",
    },
    "user_id": 182696006,
}

TEST_COMMENT_ITEM = {
    "bit_flags": 0,
    "comment_like_count": 1,
    "content_type": "comment",
    "created_at": 1494751960,
    "created_at_utc": 1494751960,
    "did_report_as_spam": False,
    "has_liked_comment": True,
    "inline_composer_display_condition": "never",
    "pk": 17856583722163490,
    "status": "Active",
    "text": "Wow awesome take!",
    "type": 0,
    "user": {
        "full_name": "Jon",
        "is_private": False,
        "is_verified": False,
        "pk": 4236956175,
        "profile_pic_id": "1699734799357160789_42369123453",
        "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/cf320d98994fasdasdas5fa53c698996bfa/5B0AD6BE/t51.2885-19/s150x150/26398129_141825939841889_6195199731287719936_n.jpg",
        "username": "test_user_name",
    },
    "user_id": 4236736455,
}

TEST_USER_ITEM = {
    "pk": 1234543212321,
    "username": "test_username",
    "full_name": "Test Username",
    "is_private": False,
    "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/6f3e8913b4e9c3153bc15669c47c9519/5AE4F003/t51.2885-19/s150x150/21827786_319483561795594_805639369123123123_n.jpg",
    "profile_pic_id": "120334920291626198135_614512764",
    "friendship_status": {
        "following": True,
        "is_private": False,
        "incoming_request": False,
        "outgoing_request": False,
        "is_bestie": False,
    },
    "is_verified": False,
    "has_anonymous_profile_picture": False,
    "follower_count": 470,
    "byline": "470 followers",
    "social_context": "Following",
    "search_social_context": "Following",
    "mutual_followers_count": 0.0,
    "unseen_count": 0,
}

TEST_USERNAME_INFO_ITEM = {
    "aggregate_promote_engagement": False,
    "allowed_commenter_type": "any",
    "auto_expand_chaining": False,
    "besties_count": 0,
    "biography": "TEST",
    "can_be_tagged_as_sponsor": False,
    "can_boost_post": False,
    "can_convert_to_business": True,
    "can_create_sponsor_tags": False,
    "can_follow_hashtag": False,
    "can_see_organic_insights": False,
    "external_url": "",
    "feed_post_reshare_disabled": False,
    "follower_count": 100,
    "following_count": 100,
    "full_name": "Test user",
    "has_anonymous_profile_picture": False,
    "has_chaining": True,
    "has_highlight_reels": False,
    "has_profile_video_feed": False,
    "hd_profile_pic_url_info": {
        "height": 1080,
        "url": "https://scontent-ams3-1.cdninstagram.com/vp/b99141f9080640b7bec84e7a67caf69e/5B27B215/t51.7777-19/17332952_229750480834181_377777777357477136_a.jpg",
        "width": 1080,
    },
    "hd_profile_pic_versions": [
        {
            "height": 320,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/d8083f7cb9ec5e0c265ed832867ec62f/5B3C2B1F/t51.7777-19/s320x320/17332952_229750487777777_330377473574363136_a.jpg",
            "width": 320,
        },
        {
            "height": 640,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/12596165258c466f05979c1be6e95b07/5B294470/t51.7777-19/s640x640/17332952_229750487777777_330377473574363136_a.jpg",
            "width": 640,
        },
    ],
    "include_direct_blacklist_status": True,
    "is_business": False,
    "is_needy": False,
    "is_private": False,
    "is_profile_action_needed": False,
    "is_verified": False,
    "is_video_creator": False,
    "media_count": 100,
    "pk": 7777777777,
    "profile_pic_id": "14779892390947777777_18277776",
    "profile_pic_url": "https://scontent-ams3-1.cdninstagram.com/vp/d056984e148d566bbb15f7eaad55304e/5B4DF6EF/t51.7777-19/s150x150/17377752_27777770480834181_3303899473577777736_a.jpg",
    "recently_bestied_by_count": 0,
    "reel_auto_archive": "unset",
    "show_besties_badge": True,
    "show_business_conversion_icon": False,
    "show_conversion_edit_entry": True,
    "show_insights_terms": False,
    "username": "test_user",
    "usertag_review_enabled": False,
    "usertags_count": 16,
}

TEST_SEARCH_USERNAME_ITEM = {
    "biography": "Some biography",
    "external_url": "",
    "follower_count": 100,
    "following_count": 100,
    "full_name": "Awesome Test User",
    "has_anonymous_profile_picture": False,
    "has_chaining": False,
    "hd_profile_pic_url_info": {
        "height": 440,
        "url": "https://scontent-ams3-1.cdninstagram.com/vp/0049430734318b5063c4cb2cee26862d/5B399C46/t51.2485-29/28435967_574234472958498_24213573932476253432_n.jpg",
        "width": 440,
    },
    "hd_profile_pic_versions": [
        {
            "height": 320,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/9a869a4fffb0f65b4d850cae433ab00c/5B3B894C/t55.25585-19/s320x320/28435967_574234472958498_24213573958234234236_n.jpg",
            "width": 320,
        }
    ],
    "is_business": False,
    "is_favorite": False,
    "is_private": False,
    "is_verified": False,
    "media_count": 100,
    "pk": 777777777,
    "profile_pic_id": "1727616424669463993_721567777777",
    "profile_pic_url": "https://scontent-ams3-1.cdninstagram.com/vp/86a59b5dadc88e6b6a723b73a6ba25a7/5B271FBC/t51.7777-19/s150x150/28435967_574234472958498_24213577777777777_n.jpg",
    "reel_auto_archive": "unset",
    "username": "awesome.test.user",
    "usertags_count": 0,
}

TEST_PHOTO_ITEM = {
    "taken_at": 1281669687,
    "pk": 1234,
    "id": "1234_19",
    "device_timestamp": 1281669538,
    "media_type": 1,
    "code": "TS",
    "client_cache_key": "MTIzNA==.2",
    "filter_type": 0,
    "image_versions2": {
        "candidates": [
            {
                "width": 612,
                "height": 612,
                "url": "https://scontent-amt2-1.cdninstagram.com/vp/9ef94dbfea2b8cdb2ba5c9b45f1945fd/5AEFE328/t51.2885-15/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2",
            },
            {
                "width": 240,
                "height": 240,
                "url": "https://scontent-amt2-1.cdninstagram.com/vp/3f4554f2e45bf356951b2a437ab7d5b1/5ADE3631/t51.2885-15/s240x240/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2",
            },
        ]
    },
    "original_width": 612,
    "original_height": 612,
    "lat": 37.3988349713,
    "lng": -122.027721405,
    "user": {
        "pk": 19,
        "username": "chris",
        "full_name": "Chris Messina",
        "is_private": False,
        "profile_pic_url": "https://scontent-amt2-1.cdninstagram.com/t51.2885-19/s150x150/23824744_1036957976446940_7940760494346862592_n.jpg",
        "profile_pic_id": "1654528278723030076_19",
        "friendship_status": {
            "following": False,
            "outgoing_request": False,
            "is_bestie": False,
        },
        "is_verified": False,
        "has_anonymous_profile_picture": False,
        "is_unpublished": False,
        "is_favorite": False,
    },
    "caption": None,
    "caption_is_edited": False,
    "like_count": 260,
    "has_liked": False,
    "top_likers": [],
    "comment_likes_enabled": True,
    "comment_threading_enabled": False,
    "has_more_comments": True,
    "max_num_visible_preview_comments": 2,
    "preview_comments": [],
    "comment_count": 18,
    "photo_of_you": False,
    "can_viewer_save": True,
    "organic_tracking_token": "eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiYmVkOTQ3ODIzODlkNDE2Nzg2ZTExNDc5ZmExMTkyYTIxMjM0Iiwic2VydmVyX3Rva2VuIjoiMTUxNjIyMTI0MzYxMXwxMjM0fDE4MjY5NjAwNnw5NGQzMjMyMmUxYzgxM2U3MWJmYTZjYzkzZjgxMTgyYzZmMzFmMGUyZTA2ODFjZjI1YzA4YzBiZDFkMWQ3M2U5In0sInNpZ25hdHVyZSI6IiJ9",
}

TEST_TIMELINE_PHOTO_ITEM = {
    "media_or_ad": {
        "taken_at": 1281669687,
        "pk": 1234,
        "id": "1234_19",
        "device_timestamp": 1281669538,
        "media_type": 1,
        "code": "TS",
        "client_cache_key": "MTIzNA==.2",
        "filter_type": 0,
        "image_versions2": {
            "candidates": [
                {
                    "width": 612,
                    "height": 612,
                    "url": "https://scontent-amt2-1.cdninstagram.com/vp/9ef94dbfea2b8cdb2ba5c9b45f1945fd/5AEFE328/t51.2885-15/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2",
                },
                {
                    "width": 240,
                    "height": 240,
                    "url": "https://scontent-amt2-1.cdninstagram.com/vp/3f4554f2e45bf356951b2a437ab7d5b1/5ADE3631/t51.2885-15/s240x240/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2",
                },
            ]
        },
        "original_width": 612,
        "original_height": 612,
        "lat": 37.3988349713,
        "lng": -122.027721405,
        "user": {
            "pk": 19,
            "username": "chris",
            "full_name": "Chris Messina",
            "is_private": False,
            "profile_pic_url": "https://scontent-amt2-1.cdninstagram.com/t51.2885-19/s150x150/23824744_1036957976446940_7940760494346862592_n.jpg",
            "profile_pic_id": "1654528278723030076_19",
            "friendship_status": {
                "following": False,
                "outgoing_request": False,
                "is_bestie": False,
            },
            "is_verified": False,
            "has_anonymous_profile_picture": False,
            "is_unpublished": False,
            "is_favorite": False,
        },
        "caption": None,
        "caption_is_edited": False,
        "like_count": 260,
        "has_liked": False,
        "top_likers": [],
        "comment_likes_enabled": True,
        "comment_threading_enabled": False,
        "has_more_comments": True,
        "max_num_visible_preview_comments": 2,
        "preview_comments": [],
        "comment_count": 18,
        "photo_of_you": False,
        "can_viewer_save": True,
        "organic_tracking_token": "eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiYmVkOTQ3ODIzODlkNDE2Nzg2ZTExNDc5ZmExMTkyYTIxMjM0Iiwic2VydmVyX3Rva2VuIjoiMTUxNjIyMTI0MzYxMXwxMjM0fDE4MjY5NjAwNnw5NGQzMjMyMmUxYzgxM2U3MWJmYTZjYzkzZjgxMTgyYzZmMzFmMGUyZTA2ODFjZjI1YzA4YzBiZDFkMWQ3M2U5In0sInNpZ25hdHVyZSI6IiJ9",
    }
}

TEST_USER_TAG_ITEM = {
    u"code": u"BRv8jaDFbsJ",
    u"can_viewer_save": True,
    u"usertags": {
        u"in": [
            {
                u"position": [0.8449074, 0.106481485],
                u"start_time_in_video_in_sec": None,
                u"user": {
                    u"username": u"iceprincezamani",
                    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/af79705613258faaa711b019f7ba7ac4/5CCEBDB3/t51.2885-19/s150x150/41606837_461088254381942_1625040398730657792_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
                    u"profile_pic_id": u"1869904537540938101_184565065",
                    u"full_name": u"Panshak Zamani :-)",
                    u"pk": 184565065,
                    u"is_verified": True,
                    u"is_private": False,
                },
                u"duration_in_video_in_sec": None,
            },
            {
                u"position": [0.6053240999999999, 0.9638888999999999],
                u"start_time_in_video_in_sec": None,
                u"user": {
                    u"username": u"xsproject",
                    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/ce16e991b80484564d2e9b8d6af70094/5CD13391/t51.2885-19/10570253_285800514958746_1157588212_a.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
                    u"full_name": u"XS PROJECT",
                    u"pk": 254488727,
                    u"is_verified": False,
                    u"is_private": False,
                },
                u"duration_in_video_in_sec": None,
            },
            {
                u"position": [0.23263889999999998, 0.7046296],
                u"start_time_in_video_in_sec": None,
                u"user": {
                    u"username": u"maxdevblock",
                    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/ff25ff3b684dea00e1918081c93c20e2/5CBA9941/t51.2885-19/s150x150/15876423_379687975701910_4035551486069964800_a.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
                    u"profile_pic_id": u"1419754651084223374_3998456661",
                    u"full_name": u"\U0001f13c\U0001f130\U0001f147\U0001f173\U0001f174\U0001f185\U0001f131\U0001f13b\U0001f13e\U0001f132\U0001f13a",
                    u"pk": 3998456661,
                    u"is_verified": False,
                    u"is_private": False,
                },
                u"duration_in_video_in_sec": None,
            },
        ]
    },
    u"next_max_id": 17853655591136137,
    u"comment_likes_enabled": True,
    u"max_num_visible_preview_comments": 2,
    u"like_count": 52,
    u"image_versions2": {
        u"candidates": [
            {
                u"url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/aa092049c99e3ad1b6a4c862e347c261/5CD1FFFF/t51.2885-15/e35/17266207_160518517800958_3004377164544999424_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net&se=7&ig_cache_key=MTQ3MjY2MTkxOTExNDgzNjc0NQ%3D%3D.2",
                u"width": 1080,
                u"height": 1350,
            },
            {
                u"url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/9ba27b08ab8a035f0151d6b9a4e2f87e/5CCD7CE4/t51.2885-15/e35/p240x240/17266207_160518517800958_3004377164544999424_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net&ig_cache_key=MTQ3MjY2MTkxOTExNDgzNjc0NQ%3D%3D.2",
                u"width": 240,
                u"height": 300,
            },
        ]
    },
    u"comment_threading_enabled": True,
    u"id": u"1472661919114836745_2308259673",
    u"preview_comments": [
        {
            u"status": u"Active",
            u"user_id": 4556478952,
            u"created_at_utc": 1489818661,
            u"created_at": 1489818661,
            u"bit_flags": 0,
            u"share_enabled": False,
            u"did_report_as_spam": False,
            u"user": {
                u"username": u"webdeveloper_world",
                u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/54a66674c1f1d5cd56b56c75e3d6ec1e/5CCA7BE8/t51.2885-19/s150x150/32233103_638476819831993_8407495672734941184_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
                u"profile_pic_id": u"1442122589220544777_4556478952",
                u"full_name": u"Webdeveloper World",
                u"pk": 4556478952,
                u"is_verified": False,
                u"is_private": False,
            },
            u"content_type": u"comment",
            u"text": u"Brilliant! Best one so far!",
            u"media_id": 1472661919114836745,
            u"pk": 17862027220087150,
            u"type": 0,
        }
    ],
    u"client_cache_key": u"MTQ3MjY2MTkxOTExNDgzNjc0NQ==.2",
    u"device_timestamp": 1489774722152,
    u"comment_count": 3,
    u"media_type": 1,
    u"organic_tracking_token": u"eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiZjBiNjg4NGY1Zjg4NDk3ZDlkYmUwZDE2OWJhNmFlZGUxNDcyNjYxOTE5MTE0ODM2NzQ1Iiwic2VydmVyX3Rva2VuIjoiMTU0NzYzMDUzNjc1N3wxNDcyNjYxOTE5MTE0ODM2NzQ1fDM5OTg0NTY2NjF8YjY0YzFmNjRmZjM2NjJhMWY0NjgzZWFiY2ZmNzUwN2U4ZDM1OGU3ZDRmOWUyODBkNTljNWYwZjdlNDZiZGFkMyJ9LCJzaWduYXR1cmUiOiIifQ==",
    u"caption_is_edited": False,
    u"inline_composer_display_condition": u"impression_trigger",
    u"original_height": 1350,
    u"filter_type": 112,
    u"user": {
        u"username": u"monsieurwinner",
        u"reel_auto_archive": u"unset",
        u"has_anonymous_profile_picture": False,
        u"is_unpublished": False,
        u"is_favorite": False,
        u"friendship_status": {
            u"following": False,
            u"outgoing_request": False,
            u"is_bestie": False,
        },
        u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/9f47c96e348c6eaaea7cf643ee626008/5CBCD374/t51.2885-19/s150x150/46915113_2288103657891057_3713770861216399360_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
        u"profile_pic_id": u"1942331692376923386_2308259673",
        u"full_name": u"MrWinner CEO Musiklibrary.com",
        u"pk": 2308259673,
        u"is_verified": False,
        u"is_private": False,
    },
    u"pk": 1472661919114836745,
    u"has_liked": True,
    u"has_more_comments": True,
    u"can_view_more_preview_comments": True,
    u"photo_of_you": True,
    u"caption": {
        u"status": u"Active",
        u"user_id": 2308259673,
        u"created_at_utc": 1489775003,
        u"created_at": 1489775003,
        u"bit_flags": 0,
        u"share_enabled": False,
        u"did_report_as_spam": False,
        u"user": {
            u"username": u"monsieurwinner",
            u"reel_auto_archive": u"unset",
            u"has_anonymous_profile_picture": False,
            u"is_unpublished": False,
            u"is_favorite": False,
            u"friendship_status": {
                u"following": False,
                u"outgoing_request": False,
                u"is_bestie": False,
            },
            u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/9f47c96e348c6eaaea7cf643ee626008/5CBCD374/t51.2885-19/s150x150/46915113_2288103657891057_3713770861216399360_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
            u"profile_pic_id": u"1942331692376923386_2308259673",
            u"full_name": u"MrWinner CEO Musiklibrary.com",
            u"pk": 2308259673,
            u"is_verified": False,
            u"is_private": False,
        },
        u"content_type": u"comment",
        u"text": u"#Classicman Programmers are not always busy to the point that, they don't care for themselves. Programmers are #cute #handsome #smart #classicman. We love #program, #code #think #build #test #debugg #run.\n#developer #winner #php #java #webdeveloper #webdesigner #femalecoder #femaleprogrammer #blackAndSmart #blackandclean",
        u"media_id": 1472661919114836745,
        u"pk": 17874973114042066,
        u"type": 1,
    },
    u"taken_at": 1489775002,
    u"original_width": 1080,
    u"can_viewer_reshare": True,
}

TEST_MEDIA_LIKER = {
    u"full_name": u"Instagrambot Instabot",
    u"is_private": False,
    u"is_verified": False,
    u"pk": 9876543210,
    u"profile_pic_id": u"1234567890123456789_123456789",
    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
    u"username": u"instabot",
}

TEST_FOLLOWER_ITEM = {
    u"full_name": u"Follower Username",
    u"has_anonymous_profile_picture": False,
    u"is_private": False,
    u"is_verified": False,
    u"pk": 1234567890,
    u"profile_pic_id": u"1234567890123456789_1234567890",
    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
    u"reel_auto_archive": u"on",
    u"username": u"follower.username.",
}

TEST_FOLLOWING_ITEM = {
    u"full_name": u"Following Username",
    u"has_anonymous_profile_picture": False,
    u"is_favorite": False,
    u"is_private": False,
    u"is_verified": False,
    u"pk": 1234567890,
    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
    u"reel_auto_archive": u"unset",
    u"username": u"following.username",
}

TEST_COMMENT_LIKER_ITEM = {
    u"full_name": u"Instagrambot Instabot",
    u"is_private": False,
    u"is_verified": False,
    u"pk": 9876543210,
    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
    u"username": u"instabot",
}

TEST_LOCATION_ITEM = {
    u"header_media": {},
    u"location": {
        u"address": u"",
        u"city": u"Location City Name",
        u"external_source": u"facebook_places",
        u"facebook_places_id": 123456789012345,
        u"lat": 1.2345,
        u"lng": 9.8765,
        u"name": u"City, State",
        u"pk": 123456789,
        u"short_name": u"City",
    },
    u"media_bundles": [],
    u"subtitle": u"",
    u"title": u"City, State",
}

TEST_MOST_RECENT_INVITER_ITEM = {
    u"full_name": u"Inbox User 1",
    u"has_anonymous_profile_picture": False,
    u"is_private": False,
    u"is_verified": False,
    u"pk": 12345678901,
    u"profile_pic_id": u"1952395551539983563_12345678901",
    u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/415ce786f601f6950cedaf19570ee0fd/5CC331E4/t51.2885-19/s150x150/49682915_326457101297245_1431392662795059200_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
    u"reel_auto_archive": u"on",
    u"username": u"inbox_user_1",
}

TEST_INBOX_THREAD_ITEM = {
    u"canonical": True,
    u"has_newer": False,
    u"has_older": True,
    u"inviter": {
        u"full_name": u"Inbox User 2",
        u"has_anonymous_profile_picture": False,
        u"is_private": False,
        u"is_verified": False,
        u"pk": 1234567890,
        u"profile_pic_id": u"1234567890123456789_1234567890",
        u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/e9c32c0f4591557f6beec9cb199a44b4/5CD22060/t51.2885-19/s150x150/12345678_293082444858234_5330756439625433088_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
        u"reel_auto_archive": u"on",
        u"username": u"inbox.user.2",
    },
    u"is_group": False,
    u"is_pin": False,
    u"is_spam": False,
    u"items": [
        {
            u"client_context": u"74a0440f-c5d1-441c-af16-d0e109fc6a3c",
            u"item_id": u"28544793378841687329460340129792000",
            u"item_type": u"text",
            u"text": u"AhHHHH cool",
            u"timestamp": 1547416349724500,
            u"user_id": 1098765432,
        }
    ],
    u"last_activity_at": 1547416349724500,
    u"last_permanent_item": {
        u"client_context": u"74a0440f-c5d1-441c-af16-d0e109fc6a3c",
        u"item_id": u"28544793378841687329460340129792000",
        u"item_type": u"text",
        u"text": u"AhHHHH cool",
        u"timestamp": 1547416349724500,
        u"user_id": 1098765432,
    },
    u"last_seen_at": {
        u"1098765432": {
            u"item_id": u"28544793378841687329460340129792000",
            u"timestamp": u"1547416349724500",
        }
    },
    u"left_users": [],
    u"mentions_muted": False,
    u"muted": False,
    u"named": False,
    u"newest_cursor": u"28544793378841687329460340129792000",
    u"oldest_cursor": u"28544793378841687329460340129792000",
    u"pending": False,
    u"pending_score": 1547416349724500,
    u"thread_id": u"340282366841710300949128210682725503544",
    u"thread_title": u"inbox.user.2",
    u"thread_type": u"private",
    u"thread_v2_id": u"17940844501242424",
    u"users": [
        {
            u"friendship_status": {
                u"blocking": False,
                u"following": True,
                u"incoming_request": False,
                u"is_bestie": False,
                u"is_private": False,
                u"outgoing_request": False,
            },
            u"full_name": u"Inbox User 2",
            u"has_anonymous_profile_picture": False,
            u"is_directapp_installed": False,
            u"is_private": False,
            u"is_verified": False,
            u"pk": 1234567890,
            u"profile_pic_id": u"1234567890123456789_1234567890",
            u"profile_pic_url": u"https://instagram.fmxp5-1.fna.fbcdn.net/vp/e9c32c0f4591557f6beec9cb199a44b4/5CD22060/t51.2885-19/s150x150/12345678_293082444858234_5330756439625433088_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net",
            u"reel_auto_archive": u"on",
            u"username": u"inbox.user.2",
        }
    ],
    u"valued_request": False,
    u"vc_muted": False,
    u"viewer_id": 1098765432,
}



import os


def upload_video(self, video, caption="", thumbnail=None, options={}):
    """Upload video to Instagram

    @param video      Path to video file (String)
    @param caption    Media description (String)
    @param thumbnail  Path to thumbnail for video (String). When None, then thumbnail is generate automatically
    @param options    Object with difference options, e.g. configure_timeout, rename_thumbnail, rename (Dict)
                      Designed to reduce the number of function arguments!

    @return           Object with state of uploading to Instagram (or False)
    """
    self.small_delay()
    self.logger.info("Started uploading '{video}'".format(video=video))
    result = self.api.upload_video(
        video, caption=caption, thumbnail=thumbnail, options=options
    )
    if not result:
        self.logger.info("Video '{}' is not {} .".format(video, "uploaded"))
        return False
    self.logger.info("Video '{video}' uploaded".format(video=video))
    return result


def download_video(
    self, media_id, folder="videos", filename=None, save_description=False
):
    self.small_delay()
    if not os.path.exists(folder):
        os.makedirs(folder)
    if save_description:
        media = self.get_media_info(media_id)[0]
        caption = media["caption"]["text"] if media["caption"] else ""
        username = media["user"]["username"]
        fname = os.path.join(folder, "{}_{}.txt".format(username, media_id))
        with open(fname, encoding="utf8", mode="w") as f:
            f.write(caption)
    try:
        return self.api.download_video(media_id, filename, False, folder)
    except Exception:
        self.logger.info("Media with `{}` is not downloaded.".format(media_id))
        return False



import pytest
import responses

from instabot.api.config import API_URL, SIG_KEY_VERSION

from .test_bot import TestBot
from .test_variables import (
    TEST_CAPTION_ITEM,
    TEST_COMMENT_ITEM,
    TEST_FOLLOWER_ITEM,
    TEST_FOLLOWING_ITEM,
    TEST_PHOTO_ITEM,
    TEST_SEARCH_USERNAME_ITEM,
    TEST_TIMELINE_PHOTO_ITEM,
    TEST_USERNAME_INFO_ITEM,
)

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class TestBotGet(TestBot):
    @pytest.mark.parametrize(
        "media_id,check_media,"
        "comment_txt,"
        "has_liked,"
        "like_count,"
        "has_anonymous_profile_picture,filter_users_without_profile_photo,"
        "expected",
        [
            (1234567890, False, False, True, float("inf"), True, True, True),
            (1234567890, False, False, True, float("inf"), True, False, True),
            (1234567890, False, False, True, float("inf"), False, True, True),
            (1234567890, False, False, True, float("inf"), False, False, True),
            (1234567890, True, False, True, float("inf"), True, True, False),
            (1234567890, True, False, True, float("inf"), True, False, False),
            (1234567890, True, False, True, float("inf"), False, True, False),
            (1234567890, True, False, True, float("inf"), False, False, False),
            (1234567890, False, False, False, float("inf"), True, True, True),
            (1234567890, False, False, False, float("inf"), True, False, True),
            (1234567890, False, False, False, float("inf"), False, True, True),
            (1234567890, False, False, False, float("inf"), False, False, True),
            (1234567890, True, False, False, float("inf"), True, True, False),
            (1234567890, True, False, False, float("inf"), True, False, False),
            (1234567890, True, False, False, float("inf"), False, True, False),
            (1234567890, True, False, False, float("inf"), False, False, False),
            (1234567890, False, False, True, False, True, True, True),
            (1234567890, False, False, True, False, True, False, True),
            (1234567890, False, False, True, False, False, True, True),
            (1234567890, False, False, True, False, False, False, True),
            (1234567890, True, False, True, False, True, True, False),
            (1234567890, True, False, True, False, True, False, False),
            (1234567890, True, False, True, False, False, True, False),
            (1234567890, True, False, True, False, False, False, False),
            (1234567890, False, False, False, False, True, True, True),
            (1234567890, False, False, False, False, True, False, True),
            (1234567890, False, False, False, False, False, True, True),
            (1234567890, False, False, False, False, False, False, True),
            (1234567890, True, False, False, False, True, True, False),
            (1234567890, True, False, False, False, True, False, True),
            (1234567890, True, False, False, False, False, True, True),
            (1234567890, True, False, False, False, False, False, True),
            (1234567890, False, True, True, float("inf"), True, True, True),
            (1234567890, False, True, True, float("inf"), True, False, True),
            (1234567890, False, True, True, float("inf"), False, True, True),
            (1234567890, False, True, True, float("inf"), False, False, True),
            (1234567890, True, True, True, float("inf"), True, True, False),
            (1234567890, True, True, True, float("inf"), True, False, False),
            (1234567890, True, True, True, float("inf"), False, True, False),
            (1234567890, True, True, True, float("inf"), False, False, False),
            (1234567890, False, True, False, float("inf"), True, True, True),
            (1234567890, False, True, False, float("inf"), True, False, True),
            (1234567890, False, True, False, float("inf"), False, True, True),
            (1234567890, False, True, False, float("inf"), False, False, True),
            (1234567890, True, True, False, float("inf"), True, True, False),
            (1234567890, True, True, False, float("inf"), True, False, False),
            (1234567890, True, True, False, float("inf"), False, True, False),
            (1234567890, True, True, False, float("inf"), False, False, False),
            (1234567890, False, True, True, False, True, True, True),
            (1234567890, False, True, True, False, True, False, True),
            (1234567890, False, True, True, False, False, True, True),
            (1234567890, False, True, True, False, False, False, True),
            (1234567890, True, True, True, False, True, True, False),
            (1234567890, True, True, True, False, True, False, False),
            (1234567890, True, True, True, False, False, True, False),
            (1234567890, True, True, True, False, False, False, False),
            (1234567890, False, True, False, False, True, True, True),
            (1234567890, False, True, False, False, True, False, True),
            (1234567890, False, True, False, False, False, True, True),
            (1234567890, False, True, False, False, False, False, True),
            (1234567890, True, True, False, False, True, True, False),
            (1234567890, True, True, False, False, True, False, False),
            (1234567890, True, True, False, False, False, True, False),
            (1234567890, True, True, False, False, False, False, False),
        ],
    )
    @responses.activate
    @patch("time.sleep", return_value=None)
    def test_bot_like(
        self,
        patched_time_sleep,
        media_id,
        check_media,
        comment_txt,
        has_liked,
        like_count,
        has_anonymous_profile_picture,
        filter_users_without_profile_photo,
        expected,
    ):

        self.bot._following = [1]
        TEST_PHOTO_ITEM["has_liked"] = has_liked
        if not like_count:
            like_count = self.bot.min_likes_to_like + 1
        TEST_PHOTO_ITEM["like_count"] = like_count
        TEST_PHOTO_ITEM["user"]["pk"] = self.bot.user_id + 1
        TEST_USERNAME_INFO_ITEM["pk"] = self.bot.user_id + 2
        TEST_USERNAME_INFO_ITEM["follower_count"] = 100
        TEST_USERNAME_INFO_ITEM["following_count"] = 15
        TEST_USERNAME_INFO_ITEM[
            "has_anonymous_profile_picture"
        ] = has_anonymous_profile_picture
        self.bot.filter_users_without_profile_photo = filter_users_without_profile_photo
        TEST_USERNAME_INFO_ITEM["is_business"] = False
        TEST_USERNAME_INFO_ITEM["is_private"] = False
        TEST_USERNAME_INFO_ITEM["is_verified"] = False
        TEST_USERNAME_INFO_ITEM["media_count"] = self.bot.min_media_count_to_follow + 1
        if comment_txt:
            comment_txt = " ".join(self.bot.blacklist_hashtags)
        else:
            comment_txt = "instabot"
        TEST_USERNAME_INFO_ITEM["biography"] = comment_txt

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=media_id
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM],
            },
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=media_id
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM],
            },
            status=200,
        )

        results = 1
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=media_id
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM],
            },
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=media_id
            ),
            status=200,
            json={"status": "ok"},
        )

        assert self.bot.like(media_id, check_media=check_media) == expected

    @pytest.mark.parametrize("comment_id", [12345678901234567, "12345678901234567"])
    @responses.activate
    def test_bot_like_comment(self, comment_id):
        responses.add(
            responses.POST,
            "{api_url}media/{comment_id}/comment_like/".format(
                api_url=API_URL, comment_id=comment_id
            ),
            json={"status": "ok"},
            status=200,
        )
        assert self.bot.like_comment(comment_id)

    @responses.activate
    @pytest.mark.parametrize(
        "has_liked_comment,comment_id",
        [(True, True), (True, False), (False, False), (False, True)],
    )
    @patch("time.sleep", return_value=None)
    def test_like_media_comments(
        self, patched_time_sleep, has_liked_comment, comment_id
    ):
        TEST_COMMENT_ITEM["has_liked_comment"] = has_liked_comment
        results = 2
        if comment_id or has_liked_comment:
            comment_id = TEST_COMMENT_ITEM["pk"]
            expected_broken_items = []
        else:
            comment_id = "wrong_comment_id"
            expected_broken_items = [TEST_COMMENT_ITEM["pk"] for _ in range(results)]
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        media_id = 1234567890
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=200,
        )
        responses.add(
            responses.POST,
            "{api_url}media/{comment_id}/comment_like/".format(
                api_url=API_URL, comment_id=comment_id
            ),
            json={"status": "ok"},
            status=200,
        )
        broken_items = self.bot.like_media_comments(media_id)
        assert broken_items == expected_broken_items

    @responses.activate
    @pytest.mark.parametrize("user_id", ["1234567890", 1234567890])
    @patch("time.sleep", return_value=None)
    def test_like_user(self, patched_time_sleep, user_id):
        self.bot._following = [1]

        TEST_USERNAME_INFO_ITEM["biography"] = "instabot"

        response_data = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=user_id
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=user_id),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=user_id),
            status=200,
            json=response_data,
        )

        results = 5
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": [TEST_PHOTO_ITEM for _ in range(results)],
        }
        responses.add(
            responses.GET,
            "{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true".format(
                api_url=API_URL,
                user_id=user_id,
                max_id="",
                min_timestamp=None,
                rank_token=self.bot.api.rank_token,
            ),
            json=response_data,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM],
            },
            status=200,
        )

        results = 2
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            status=200,
            json={"status": "ok"},
        )

        broken_items = self.bot.like_user(user_id)
        assert [] == broken_items

    @responses.activate
    @pytest.mark.parametrize("user_ids", [["1234567890"], [1234567890]])
    @patch("time.sleep", return_value=None)
    def test_like_users(self, patched_time_sleep, user_ids):

        self.bot._following = [1]

        TEST_USERNAME_INFO_ITEM["biography"] = "instabot"

        response_data = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=user_ids[0]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=user_ids[0]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=user_ids[0]
            ),
            status=200,
            json=response_data,
        )

        results_1 = 5
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results_1,
            "status": "ok",
            "more_available": False,
            "items": [TEST_PHOTO_ITEM for _ in range(results_1)],
        }
        responses.add(
            responses.GET,
            "{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true".format(
                api_url=API_URL,
                user_id=user_ids[0],
                max_id="",
                min_timestamp=None,
                rank_token=self.bot.api.rank_token,
            ),
            json=response_data,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM],
            },
            status=200,
        )

        results_2 = 2
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results_2,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results_2)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            status=200,
            json={"status": "ok"},
        )

        self.bot.like_users(user_ids)
        assert self.bot.total["likes"] == results_1

    @responses.activate
    @pytest.mark.parametrize(
        "blocked_actions_protection,blocked_actions",
        [(True, True), (True, False), (False, True), (False, True)],
    )
    @patch("time.sleep", return_value=None)
    def test_like_feedback(
        self, patched_time_sleep, blocked_actions_protection, blocked_actions
    ):
        self.bot.blocked_actions_protection = blocked_actions_protection
        self.bot.blocked_actions["likes"] = blocked_actions
        media_id = 1234567890
        response_data = {
            u"status": u"fail",
            u"feedback_title": u"You\u2019re Temporarily Blocked",
            u"feedback_message": u"It looks like you were misusing this feature by going too fast. You\u2019ve been temporarily blocked from using it. We restrict certain content and actions to protect our community. Tell us if you think we made a mistake.",
            u"spam": True,
            u"feedback_action": u"report_problem",
            u"feedback_appeal_label": u"Report problem",
            u"feedback_ignore_label": u"OK",
            u"message": u"feedback_required",
            u"feedback_url": u"repute/report_problem/instagram_like_add/",
        }
        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=400,
        )
        assert not self.bot.like(media_id, check_media=False)

    @responses.activate
    @pytest.mark.parametrize("medias", [[1234567890, 9876543210]])
    @patch("time.sleep", return_value=None)
    def test_like_medias(self, patched_time_sleep, medias):
        self.bot._following = [1]

        for media in medias:
            TEST_PHOTO_ITEM["id"] = media
            responses.add(
                responses.GET,
                "{api_url}media/{media_id}/info/".format(
                    api_url=API_URL, media_id=media
                ),
                json={
                    "auto_load_more_enabled": True,
                    "num_results": 1,
                    "status": "ok",
                    "more_available": False,
                    "items": [TEST_PHOTO_ITEM],
                },
                status=200,
            )

            results = 2
            response_data = {
                "caption": TEST_CAPTION_ITEM,
                "caption_is_edited": False,
                "comment_count": 4,
                "comment_likes_enabled": True,
                "comments": [TEST_COMMENT_ITEM for _ in range(results)],
                "has_more_comments": False,
                "has_more_headload_comments": False,
                "media_header_display": "none",
                "preview_comments": [],
                "status": "ok",
            }
            responses.add(
                responses.GET,
                "{api_url}media/{media_id}/comments/?".format(
                    api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
                ),
                json=response_data,
                status=200,
            )

            response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
            responses.add(
                responses.GET,
                "{api_url}users/{user_id}/info/".format(
                    api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
                ),
                status=200,
                json=response_data,
            )

            response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
            responses.add(
                responses.GET,
                "{api_url}users/{user_id}/info/".format(
                    api_url=API_URL, user_id=TEST_PHOTO_ITEM["user"]["pk"]
                ),
                status=200,
                json=response_data,
            )

            responses.add(
                responses.POST,
                "{api_url}media/{media_id}/like/".format(
                    api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
                ),
                status=200,
                json={"status": "ok"},
            )

        broken_items = self.bot.like_medias(medias)
        assert [] == broken_items

    @responses.activate
    @pytest.mark.parametrize("hashtag", ["like_hashtag1", "like_hashtag2"])
    @patch("time.sleep", return_value=None)
    def test_like_hashtag(self, patche_time_sleep, hashtag):
        self.bot._following = [1]
        liked_at_start = self.bot.total["likes"]
        results_1 = 10
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_item["has_liked"] = False
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results_1,
            "status": "ok",
            "more_available": True,
            "next_max_id": my_test_photo_item["id"],
            "items": [my_test_photo_item for _ in range(results_1)],
        }
        responses.add(
            responses.GET,
            "{api_url}feed/tag/{hashtag}/?max_id={max_id}&rank_token={rank_token}&ranked_content=true&".format(
                api_url=API_URL,
                hashtag=hashtag,
                max_id="",
                rank_token=self.bot.api.rank_token,
            ),
            json=response_data,
            status=200,
        )

        response_tag = {
            "results": [
                {
                    "id": 17841563287125205,
                    "name": hashtag,
                    "media_count": 7645915,
                    "follow_status": None,
                    "following": None,
                    "allow_following": None,
                    "allow_muting_story": None,
                    "profile_pic_url": "https://instagram.fmxp6-1.fna.fbcdn.net/vp/8e512ee62d218765d3ac46f3da6869de/5E0E0DE3/t51.2885-15/e35/c148.0.889.889a/s150x150/67618693_2467437380156007_7054420538339677194_n.jpg?_nc_ht=instagram.fmxp6-1.fna.fbcdn.net&ig_cache_key=MjExMzI5MDMwNDYxNzY3MDExMQ%3D%3D.2.c",
                    "non_violating": None,
                    "related_tags": None,
                    "subtitle": None,
                    "social_context": None,
                    "social_context_profile_links": None,
                    "follow_button_text": None,
                    "show_follow_drop_down": None,
                    "formatted_media_count": "7.6M",
                    "debug_info": None,
                    "search_result_subtitle": "7.6M posts",
                }
            ]
        }

        responses.add(
            responses.GET,
            "{api_url}tags/search/?is_typeahead=true&q={query}&rank_token={rank_token}".format(
                api_url=API_URL, query=hashtag, rank_token=self.bot.api.rank_token
            ),
            json=response_tag,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [my_test_photo_item],
            },
            status=200,
        )

        results_2 = 2
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results_2,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results_2)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM["id"]
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            status=200,
            json={"status": "ok"},
        )

        broken_items = self.bot.like_hashtag(hashtag)
        assert [] == broken_items
        assert self.bot.total["likes"] == liked_at_start + results_1

    @responses.activate
    @pytest.mark.parametrize("username", ["1234567890", 1234567890])
    @patch("time.sleep", return_value=None)
    def test_like_followers(self, patched_time_sleep, username):

        liked_at_start = self.bot.total["likes"]

        test_username = "test.username"

        response_data_1 = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=test_username
            ),
            status=200,
            json=response_data_1,
        )

        response_data_2 = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data_2,
        )

        results_3 = 2
        response_data_3 = {
            "status": "ok",
            "big_list": False,
            "next_max_id": None,
            "sections": None,
            "users": [TEST_FOLLOWER_ITEM for _ in range(results_3)],
        }
        responses.add(
            responses.GET,
            "{api_url}friendships/{user_id}/followers/?rank_token={rank_token}".format(
                api_url=API_URL, user_id=username, rank_token=self.bot.api.rank_token
            ),
            json=response_data_3,
            status=200,
        )

        self.bot._following = [1]

        TEST_USERNAME_INFO_ITEM["biography"] = "instabot"
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_item["has_liked"] = False

        response_data = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=username
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data,
        )

        results_4 = 3
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results_4,
            "status": "ok",
            "more_available": False,
            "items": [my_test_photo_item for _ in range(results_4)],
        }
        responses.add(
            responses.GET,
            "{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true".format(
                api_url=API_URL,
                user_id=username,
                max_id="",
                min_timestamp=None,
                rank_token=self.bot.api.rank_token,
            ),
            json=response_data,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [my_test_photo_item],
            },
            status=200,
        )

        results_5 = 2
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results_5,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results_5)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            status=200,
            json={"status": "ok"},
        )

        self.bot.like_followers(username)
        assert self.bot.total["likes"] == liked_at_start + results_3 * results_4

    @responses.activate
    @pytest.mark.parametrize("username", ["1234567890", 1234567890])
    @patch("time.sleep", return_value=None)
    def test_like_following(self, patched_time_sleep, username):

        liked_at_start = self.bot.total["likes"]

        test_username = "test.username"

        response_data_1 = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=test_username
            ),
            status=200,
            json=response_data_1,
        )

        response_data_2 = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data_2,
        )

        results_3 = 5
        response_data_3 = {
            "status": "ok",
            "big_list": False,
            "next_max_id": None,
            "sections": None,
            "users": [TEST_FOLLOWING_ITEM for _ in range(results_3)],
        }
        responses.add(
            responses.GET,
            "{api_url}friendships/{user_id}/following/?max_id={max_id}&ig_sig_key_version={sig_key}&rank_token={rank_token}".format(
                api_url=API_URL,
                user_id=username,
                rank_token=self.bot.api.rank_token,
                sig_key=SIG_KEY_VERSION,
                max_id="",
            ),
            json=response_data_3,
            status=200,
        )

        self.bot._following = [1]

        TEST_USERNAME_INFO_ITEM["biography"] = "instabot"
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_item["has_liked"] = False

        response_data = {"status": "ok", "user": TEST_SEARCH_USERNAME_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{username}/usernameinfo/".format(
                api_url=API_URL, username=username
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(api_url=API_URL, user_id=username),
            status=200,
            json=response_data,
        )

        results_4 = 3
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results_4,
            "status": "ok",
            "more_available": False,
            "items": [my_test_photo_item for _ in range(results_4)],
        }
        responses.add(
            responses.GET,
            "{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true".format(
                api_url=API_URL,
                user_id=username,
                max_id="",
                min_timestamp=None,
                rank_token=self.bot.api.rank_token,
            ),
            json=response_data,
            status=200,
        )

        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [my_test_photo_item],
            },
            status=200,
        )

        results_5 = 2
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results_5,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results_5)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        response_data = {"status": "ok", "user": TEST_USERNAME_INFO_ITEM}
        responses.add(
            responses.GET,
            "{api_url}users/{user_id}/info/".format(
                api_url=API_URL, user_id=my_test_photo_item["user"]["pk"]
            ),
            status=200,
            json=response_data,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL, media_id=my_test_photo_item["id"]
            ),
            status=200,
            json={"status": "ok"},
        )

        self.bot.like_following(username)
        assert self.bot.total["likes"] == liked_at_start + results_3 * results_4

    @responses.activate
    @patch("time.sleep", return_value=None)
    def test_like_timeline(self, patched_time_sleep):

        my_test_timelime_photo_item = TEST_TIMELINE_PHOTO_ITEM.copy()
        my_test_timelime_photo_item["media_or_ad"]["like_count"] = (
            self.bot.max_likes_to_like - 1
        )
        my_test_timelime_photo_item["media_or_ad"]["has_liked"] = False

        liked_at_start = self.bot.total["likes"]

        results_1 = 8
        responses.add(
            responses.POST,
            "{api_url}feed/timeline/".format(api_url=API_URL),
            json={
                "auto_load_more_enabled": True,
                "num_results": results_1,
                "is_direct_v2_enabled": True,
                "status": "ok",
                "next_max_id": None,
                "more_available": False,
                "feed_items": [my_test_timelime_photo_item for _ in range(results_1)],
            },
            status=200,
        )

        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/like/".format(
                api_url=API_URL,
                media_id=my_test_timelime_photo_item["media_or_ad"]["id"],
            ),
            status=200,
            json={"status": "ok"},
        )

        broken_items = self.bot.like_timeline()
        assert [] == broken_items
        assert self.bot.total["likes"] == liked_at_start + results_1



from __future__ import unicode_literals

import imghdr
import os
import shutil
import struct
import time

from requests_toolbelt import MultipartEncoder

from . import config


def download_photo(self, media_id, filename, media=False, folder="photos"):
    if not media:
        self.media_info(media_id)
        if not self.last_json.get("items"):
            return True
        media = self.last_json["items"][0]
    if media["media_type"] == 2:
        return True
    elif media["media_type"] == 1:
        filename = (
            "{}_{}.jpg".format(media["user"]["username"], media_id)
            if not filename
            else "{}.jpg".format(filename)
        )
        images = media["image_versions2"]["candidates"]
        fname = os.path.join(folder, filename)
        if os.path.exists(fname):
            self.logger.info("File already esists, skipping...")
            return os.path.abspath(fname)
        response = self.session.get(images[0]["url"], stream=True)
        if response.status_code == 200:
            with open(fname, "wb") as f:
                response.raw.decode_content = True
                shutil.copyfileobj(response.raw, f)
            return os.path.abspath(fname)
    else:
        success = False
        video_included = False
        for index in range(len(media["carousel_media"])):
            if media["carousel_media"][index]["media_type"] != 1:
                video_included = True
                continue
            filename_i = (
                "{}_{}_{}.jpg".format(media["user"]["username"], media_id, index)
                if not filename
                else "{}_{}.jpg".format(filename, index)
            )
            images = media["carousel_media"][index]["image_versions2"]["candidates"]
            fname = os.path.join(folder, filename_i)
            if os.path.exists(fname):
                return os.path.abspath(fname)
            response = self.session.get(images[0]["url"], stream=True)
            if response.status_code == 200:
                success = True
                with open(fname, "wb") as f:
                    response.raw.decode_content = True
                    shutil.copyfileobj(response.raw, f)
        if success:
            return os.path.abspath(fname)
        elif video_included:
            return True


def compatible_aspect_ratio(size):
    min_ratio, max_ratio = 4.0 / 5.0, 90.0 / 47.0
    width, height = size
    ratio = width * 1.0 / height * 1.0
    print("FOUND: w:{} h:{} r:{}".format(width, height, ratio))
    return min_ratio <= ratio <= max_ratio


def configure_photo(self, upload_id, photo, caption=""):
    (w, h) = get_image_size(photo)
    data = self.json_data(
        {
            "media_folder": "Instagram",
            "source_type": 4,
            "caption": caption,
            "upload_id": upload_id,
            "device": self.device_settings,
            "edits": {
                "crop_original_size": [w * 1.0, h * 1.0],
                "crop_center": [0.0, 0.0],
                "crop_zoom": 1.0,
            },
            "extra": {"source_width": w, "source_height": h},
        }
    )
    return self.send_request("media/configure/?", data)


def upload_photo(
    self,
    photo,
    caption=None,
    upload_id=None,
    from_video=False,
    force_resize=False,
    options={},
):
    """Upload photo to Instagram

    @param photo         Path to photo file (String)
    @param caption       Media description (String)
    @param upload_id     Unique upload_id (String). When None, then generate automatically
    @param from_video    A flag that signals whether the photo is loaded from the video or by itself
                         (Boolean, DEPRECATED: not used)
    @param force_resize  Force photo resize (Boolean)
    @param options       Object with difference options, e.g. configure_timeout, rename (Dict)
                         Designed to reduce the number of function arguments!
                         This is the simplest request object.

    @return Boolean
    """
    options = dict({"configure_timeout": 15, "rename": True}, **(options or {}))
    if upload_id is None:
        upload_id = str(int(time.time() * 1000))
    if not photo:
        return False
    if not compatible_aspect_ratio(get_image_size(photo)):
        self.logger.error("Photo does not have a compatible photo aspect ratio.")
        if force_resize:
            photo = resize_image(photo)
        else:
            return False

    with open(photo, "rb") as f:
        photo_bytes = f.read()

    data = {
        "upload_id": upload_id,
        "_uuid": self.uuid,
        "_csrftoken": self.token,
        "image_compression": '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
        "photo": (
            "pending_media_%s.jpg" % upload_id,
            photo_bytes,
            "application/octet-stream",
            {"Content-Transfer-Encoding": "binary"},
        ),
    }
    m = MultipartEncoder(data, boundary=self.uuid)
    self.session.headers.update(
        {
            "X-IG-Capabilities": "3Q4=",
            "X-IG-Connection-Type": "WIFI",
            "Cookie2": "$Version=1",
            "Accept-Language": "en-US",
            "Accept-Encoding": "gzip, deflate",
            "Content-type": m.content_type,
            "Connection": "close",
            "User-Agent": self.user_agent,
        }
    )
    response = self.session.post(config.API_URL + "upload/photo/", data=m.to_string())

    configure_timeout = options.get("configure_timeout")
    if response.status_code == 200:
        for attempt in range(4):
            if configure_timeout:
                time.sleep(configure_timeout)
            if self.configure_photo(upload_id, photo, caption):
                media = self.last_json.get("media")
                self.expose()
                if options.get("rename"):
                    from os import rename

                    rename(photo, "{}.REMOVE_ME".format(photo))
                return media
    return False


def get_image_size(fname):
    with open(fname, "rb") as fhandle:
        head = fhandle.read(24)
        if len(head) != 24:
            raise RuntimeError("Invalid Header")

        if imghdr.what(fname) == "png":
            check = struct.unpack(">i", head[4:8])[0]
            if check != 0x0D0A1A0A:
                raise RuntimeError("PNG: Invalid check")
            width, height = struct.unpack(">ii", head[16:24])
        elif imghdr.what(fname) == "gif":
            width, height = struct.unpack("<HH", head[6:10])
        elif imghdr.what(fname) == "jpeg":
            fhandle.seek(0)  # Read 0xff next
            size = 2
            ftype = 0
            while not 0xC0 <= ftype <= 0xCF:
                fhandle.seek(size, 1)
                byte = fhandle.read(1)
                while ord(byte) == 0xFF:
                    byte = fhandle.read(1)
                ftype = ord(byte)
                size = struct.unpack(">H", fhandle.read(2))[0] - 2
            # We are at a SOFn block
            fhandle.seek(1, 1)  # Skip `precision' byte.
            height, width = struct.unpack(">HH", fhandle.read(4))
        else:
            raise RuntimeError("Unsupported format")
        return width, height


def resize_image(fname):
    from math import ceil

    try:
        from PIL import Image, ExifTags
    except ImportError as e:
        print("ERROR: {}".format(e))
        print(
            "Required module `PIL` not installed\n"
            "Install with `pip install Pillow` and retry"
        )
        return False
    print("Analizing `{}`".format(fname))
    h_lim = {"w": 90.0, "h": 47.0}
    v_lim = {"w": 4.0, "h": 5.0}
    img = Image.open(fname)
    (w, h) = img.size
    deg = 0
    try:
        for orientation in ExifTags.TAGS.keys():
            if ExifTags.TAGS[orientation] == "Orientation":
                break
        exif = dict(img._getexif().items())
        o = exif[orientation]
        if o == 3:
            deg = 180
        if o == 6:
            deg = 270
        if o == 8:
            deg = 90
        if deg != 0:
            print("Rotating by {d} degrees".format(d=deg))
            img = img.rotate(deg, expand=True)
            (w, h) = img.size
    except (AttributeError, KeyError, IndexError) as e:
        print("No exif info found (ERR: {})".format(e))
        pass
    img = img.convert("RGBA")
    ratio = w * 1.0 / h * 1.0
    print("FOUND w:{w}, h:{h}, ratio={r}".format(w=w, h=h, r=ratio))
    if w > h:
        print("Horizontal image")
        if ratio > (h_lim["w"] / h_lim["h"]):
            print("Cropping image")
            cut = int(ceil((w - h * h_lim["w"] / h_lim["h"]) / 2))
            left = cut
            right = w - cut
            top = 0
            bottom = h
            img = img.crop((left, top, right, bottom))
            (w, h) = img.size
        if w > 1080:
            print("Resizing image")
            nw = 1080
            nh = int(ceil(1080.0 * h / w))
            img = img.resize((nw, nh), Image.ANTIALIAS)
    elif w < h:
        print("Vertical image")
        if ratio < (v_lim["w"] / v_lim["h"]):
            print("Cropping image")
            cut = int(ceil((h - w * v_lim["h"] / v_lim["w"]) / 2))
            left = 0
            right = w
            top = cut
            bottom = h - cut
            img = img.crop((left, top, right, bottom))
            (w, h) = img.size
        if h > 1080:
            print("Resizing image")
            nw = int(ceil(1080.0 * w / h))
            nh = 1080
            img = img.resize((nw, nh), Image.ANTIALIAS)
    else:
        print("Square image")
        if w > 1080:
            print("Resizing image")
            img = img.resize((1080, 1080), Image.ANTIALIAS)
    (w, h) = img.size
    new_fname = "{}.CONVERTED.jpg".format(fname)
    print("Saving new image w:{w} h:{h} to `{f}`".format(w=w, h=h, f=new_fname))
    new = Image.new("RGB", img.size, (255, 255, 255))
    new.paste(img, (0, 0, w, h), img)
    new.save(new_fname, quality=95)
    return new_fname


def stories_shaper(fname):
    """
    Find out the size of the uploaded image.
    Processing is not needed if the image is already 1080x1920 pixels.
    Otherwise, the image height should be 1920 pixels.
    Substrate formation: Crop the image under 1080x1920 pixels and apply a Gaussian Blur filter.
    Centering the image depending on its aspect ratio and paste it onto the substrate.
    Save the image.
    """
    try:
        from PIL import Image, ImageFilter
    except ImportError as e:
        print("ERROR: {}".format(e))
        print(
            "Required module `PIL` not installed\n"
            "Install with `pip install Pillow` and retry"
        )
        return False
    img = Image.open(fname)
    if (img.size[0], img.size[1]) == (1080, 1920):
        print("Image is already 1080x1920. Just converting image.")
        new_fname = "{}.STORIES.jpg".format(fname)
        new = Image.new("RGB", (img.size[0], img.size[1]), (255, 255, 255))
        new.paste(img, (0, 0, img.size[0], img.size[1]))
        new.save(new_fname)
        return new_fname
    else:
        min_width = 1080
        min_height = 1920
        if img.size[1] != 1920:
            height_percent = min_height / float(img.size[1])
            width_size = int(float(img.size[0]) * float(height_percent))
            img = img.resize((width_size, min_height), Image.ANTIALIAS)
        else:
            pass
        if img.size[0] < 1080:
            width_percent = min_width / float(img.size[0])
            height_size = int(float(img.size[1]) * float(width_percent))
            img_bg = img.resize((min_width, height_size), Image.ANTIALIAS)
        else:
            pass
        img_bg = img.crop(
            (
                int((img.size[0] - 1080) / 2),
                int((img.size[1] - 1920) / 2),
                int(1080 + ((img.size[0] - 1080) / 2)),
                int(1920 + ((img.size[1] - 1920) / 2)),
            )
        ).filter(ImageFilter.GaussianBlur(100))
        if img.size[1] > img.size[0]:
            height_percent = min_height / float(img.size[1])
            width_size = int(float(img.size[0]) * float(height_percent))
            img = img.resize((width_size, min_height), Image.ANTIALIAS)
            if img.size[0] > 1080:
                width_percent = min_width / float(img.size[0])
                height_size = int(float(img.size[1]) * float(width_percent))
                img = img.resize((min_width, height_size), Image.ANTIALIAS)
                img_bg.paste(
                    img, (int(540 - img.size[0] / 2), int(960 - img.size[1] / 2))
                )
            else:
                img_bg.paste(img, (int(540 - img.size[0] / 2), 0))
        else:
            width_percent = min_width / float(img.size[0])
            height_size = int(float(img.size[1]) * float(width_percent))
            img = img.resize((min_width, height_size), Image.ANTIALIAS)
            img_bg.paste(img, (int(540 - img.size[0] / 2), int(960 - img.size[1] / 2)))
        new_fname = "{}.STORIES.jpg".format(fname)
        print(
            "Saving new image w:{w} h:{h} to `{f}`".format(
                w=img_bg.size[0], h=img_bg.size[1], f=new_fname
            )
        )
        new = Image.new("RGB", (img_bg.size[0], img_bg.size[1]), (255, 255, 255))
        new.paste(img_bg, (0, 0, img_bg.size[0], img_bg.size[1]))
        new.save(new_fname)
        return new_fname



import pytest
import responses

from instabot.api.config import API_URL

from .test_bot import TestBot
from .test_variables import TEST_CAPTION_ITEM, TEST_COMMENT_ITEM

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class TestBotGet(TestBot):
    @responses.activate
    @pytest.mark.parametrize(
        "blocked_actions_protection,blocked_actions",
        [(True, True), (True, False), (False, True), (False, False)],
    )
    @patch("time.sleep", return_value=None)
    def test_comment_feedback(
        self, patched_time_sleep, blocked_actions_protection, blocked_actions
    ):
        self.bot.blocked_actions_protection = blocked_actions_protection
        self.bot.blocked_actions["comments"] = blocked_actions
        media_id = 1234567890
        comment_txt = "Yeah great!"

        TEST_COMMENT_ITEM["user"]["pk"] = self.bot.user_id + 1

        results = 3
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=200,
        )

        response_data = {
            "message": "feedback_required",
            "spam": True,
            "feedback_title": "Sorry, this feature isn't available right now",
            "feedback_message": "An error occurred while processing this request. Please try again later. We restrict certain content and actions to protect our community. Tell us if you think we made a mistake.",
            "feedback_url": "repute/report_problem/instagram_comment/",
            "feedback_appeal_label": "Report problem",
            "feedback_ignore_label": "OK",
            "feedback_action": "report_problem",
            "status": "fail",
        }
        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/comment/".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=400,
        )

        assert not self.bot.comment(media_id, comment_txt)

    @responses.activate
    @pytest.mark.parametrize(
        "blocked_actions_protection,blocked_actions", [(True, False), (False, False)]
    )
    @patch("time.sleep", return_value=None)
    def test_comment(
        self, patched_time_sleep, blocked_actions_protection, blocked_actions
    ):
        self.bot.blocked_actions_protection = blocked_actions_protection
        self.bot.blocked_actions["comments"] = blocked_actions
        media_id = 1234567890
        comment_txt = "Yeah great!"

        TEST_COMMENT_ITEM["user"]["pk"] = self.bot.user_id + 1

        results = 3
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok",
        }
        responses.add(
            responses.GET,
            "{api_url}media/{media_id}/comments/?".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=200,
        )

        response_data = {"status": "ok"}
        responses.add(
            responses.POST,
            "{api_url}media/{media_id}/comment/".format(
                api_url=API_URL, media_id=media_id
            ),
            json=response_data,
            status=200,
        )

        assert self.bot.comment(media_id, comment_txt)



# flake8: noqa
"""
Configuration file of the project
"""
# Config variables taken from
# https://github.com/ping/instagram_private_api/blob/master/instagram_private_api/constants.py
API_URL = "https://i.instagram.com/api/v1/"
USER_AGENT_BASE = (
    "Instagram {instagram_version} "
    "Android ({android_version}/{android_release}; "
    "{dpi}; {resolution}; {manufacturer}; "
    "{device}; {model}; {cpu}; en_US)"
)
SIG_KEY_VERSION = "4"
# version 105.0.0.18.119 (166149665)
# August 6, 2019
# from https://www.apkmirror.com/apk/instagram/instagram-instagram/instagram-instagram-105-0-0-18-119-release/instagram-105-0-0-18-119-4-android-apk-download/
# Check website regularly to update IG_SIG_KEY and X-IG-Capabilities in REQUEST_HEADERS
IG_SIG_KEY = "5f3e50f435583c9ae626302a71f7340044087a7e2c60adacfc254205a993e305"

# Request variables taken from
# https://github.com/ping/instagram_private_api/blob/422d61f0a8cc9de3d5a0e78bcba53751c44e5d63/instagram_private_api/client.py#L375
REQUEST_HEADERS = {
    "Connection": "Keep-Alive",
    # version 105.0.0.18.119 (166149665)
    # python3 -c "import base64, struct; print(base64.b64encode(struct.pack('<i', 166149665)).decode('ascii'));"
    "X-IG-Capabilities": "IT7nCQ==",
    "X-IG-App-ID": "567067343352427",
    "X-IG-Connection-Type": "WIFI",
    "X-IG-Prefetch-Request": "foreground",
    "X-IG-VP9-Capable": "false",
    "X-FB-HTTP-Engine": "Liger",
    "Accept": "*/*",
    "Accept-Encoding": "gzip,deflate",
    "Accept-Language": "en-US",
    "Content-type": "application/x-www-form-urlencoded; charset=UTF-8",
    "Cookie2": "$Version=1",
}

# Moved to the end as it's quite long
EXPERIMENTS = "ig_android_sticker_search_explorations,android_ig_camera_ar_asset_manager_improvements_universe,ig_android_stories_seen_state_serialization,ig_stories_photo_time_duration_universe,ig_android_bitmap_cache_executor_size,ig_android_stories_music_search_typeahead,ig_android_delayed_comments,ig_android_switch_back_option,ig_android_video_profiler_loom_traces,ig_android_paid_branded_content_rendering,ig_android_direct_app_reel_grid_search,ig_android_stories_no_inflation_on_app_start,ig_android_camera_sdk_check_gl_surface_r2,ig_promote_review_screen_title_universe,ig_android_direct_newer_single_line_composer_universe,ig_direct_holdout_h1_2019,ig_explore_2019_h1_destination_cover,ig_android_direct_stories_in_direct_inbox,ig_fb_graph_differentiation_no_fb_data,ig_android_recyclerview_binder_group_enabled_universe,ig_android_direct_share_sheet_custom_fast_scroller,ig_android_video_exoplayer_2,ig_android_shopping_channel_in_explore,ig_android_stories_music_filters,ig_android_2018_h1_hashtag_report_universe,ig_android_live_replay_highlights_universe,ig_android_hashtag_page_reduced_related_items,ig_android_live_titles_broadcaster_side_create_title_universe,ig_android_fbns_preload_direct_universe,ig_android_prefetch_carousels_on_swipe_universe,ig_camera_network_activity_logger,ig_camera_remove_display_rotation_cb_universe,ig_android_interactions_migrate_inline_composer_to_viewpoint_universe,ig_android_realtime_always_start_connection_on_condition_universe,ig_android_ad_leadgen_single_screen_universe,ig_android_enable_zero_rating,ig_android_import_page_post_after_biz_conversion,ig_camera_ar_effect_attribution_position,ig_android_vc_call_ended_cleanup_universe,ig_stories_engagement_holdout_2019_h1_universe,ig_android_story_import_intent,ig_direct_report_conversation_universe,ig_biz_graph_connection_universe,ig_android_codec_high_profile,ig_android_nametag,ig_android_sso_family_key_universe,ig_android_parse_direct_messages_bytes_universe,ig_hashtag_creation_universe,ig_android_gallery_order_by_date_taken,ig_android_igtv_reshare,ig_end_of_feed_universe,ig_android_share_others_post_reorder,ig_android_additional_contact_in_nux,ig_android_live_use_all_preview_sizes,ig_android_clarify_invite_options,ig_android_live_align_by_2_universe,ig_android_separate_network_executor,ig_android_realtime_manager_optimization,ig_android_auto_advance_su_unit_when_scrolled_off_screen,ig_android_network_cancellation,ig_android_media_as_sticker,ig_android_stories_video_prefetch_kb,ig_android_maintabfragment,ig_inventory_connections,ig_stories_injection_tool_enabled_universe,ig_android_stories_disable_highlights_media_preloading,ig_android_live_start_broadcast_optimized_universe,ig_android_stories_question_response_mutation_universe,ig_android_onetap_upsell_change_pwd,ig_nametag_data_collection,ig_android_disable_scroll_listeners,ig_android_persistent_nux,ig_android_igtv_audio_always_on,ig_android_enable_liger_preconnect_universe,ig_android_persistent_duplicate_notif_checker_user_based,ig_android_rate_limit_mediafeedviewablehelper,ig_android_search_remove_null_state_sections,ig_android_stories_viewer_drawable_cache_universe,ig_direct_android_reply_modal_universe,ig_android_biz_qp_suggest_page,ig_shopping_indicator_content_variations_android,ig_android_stories_reel_media_item_automatic_retry,ig_fb_notification_universe,ig_android_live_disable_speed_test_ui_timeout_universe,ig_android_direct_thread_scroll_perf_oncreate_universe,ig_android_low_data_mode_backup_2,ig_android_invite_xout_universe,ig_android_low_data_mode_backup_3,ig_android_low_data_mode_backup_4,ig_android_low_data_mode_backup_5,ig_android_video_abr_universe,ig_android_low_data_mode_backup_1,ig_android_signup_refactor_santity,ig_challenge_general_v2,ig_android_place_signature_universe,ig_android_hide_button_for_invite_facebook_friends,ig_android_business_promote_tooltip,ig_android_follow_requests_ui_improvements,ig_android_shopping_post_tagging_nux_universe,ig_android_stories_sensitivity_screen,ig_android_camera_arengine_shader_caching_universe,ig_android_insta_video_broadcaster_infra_perf,ig_android_direct_view_more_qe,ig_android_direct_visual_message_prefetch_count_universe,ig_camera_android_ar_effect_stories_deeplink,ig_android_client_side_delivery_universe,ig_android_stories_send_client_reels_on_tray_fetch_universe,ig_android_direct_inbox_background_view_models,ig_android_startup_thread_priority,ig_android_stories_viewer_responsiveness_universe,ig_android_live_use_rtc_upload_universe,ig_android_live_ama_viewer_universe,ig_android_business_id_conversion_universe,ig_smb_ads_holdout_2018_h2_universe,ig_android_modal_activity_no_animation_fix_universe,ig_android_camera_post_smile_low_end_universe,ig_android_live_realtime_comments_universe,ig_android_vc_in_app_notification_universe,ig_eof_caboose_universe,ig_android_new_one_tap_nux_universe,ig_android_igds_edit_profile_fields,ig_android_downgrade_viewport_exit_behavior,ig_android_mi_batch_upload_universe,ig_camera_android_segmentation_async_universe,ig_android_use_recyclerview_for_direct_search_universe,ig_android_live_comment_fetch_frequency_universe,ig_android_create_page_on_top_universe,ig_android_direct_log_badge_count_inconsistent,ig_android_stories_text_format_emphasis,ig_android_question_sticker_replied_state,ig_android_ad_connection_manager_universe,ig_android_image_upload_skip_queue_only_on_wifi,ig_android_ad_watchbrowse_carousel_universe,ig_android_interactions_show_verified_badge_for_preview_comments_universe,ig_stories_question_sticker_music_format_prompt,ig_android_activity_feed_row_click,ig_android_hide_crashing_newsfeed_story_t38131972,ig_android_video_upload_quality_qe1,ig_android_save_collaborative_collections,ig_android_location_attribution_text,ig_camera_android_profile_ar_notification_universe,coupon_price_test_boost_instagram_media_acquisition_universe,ig_android_video_outputsurface_handlerthread_universe,ig_android_country_code_fix_universe,ig_perf_android_holdout_2018_h1,ig_android_stories_music_overlay,ig_android_enable_lean_crash_reporting_universe,ig_android_resumable_downloads_logging_universe,ig_android_stories_default_rear_camera_universe,ig_android_low_latency_consumption_universe,ig_android_offline_mode_holdout,ig_android_foreground_location_collection,ig_android_stories_close_friends_disable_first_time_badge,ig_android_react_native_universe_kill_switch,ig_android_video_ta_universe,ig_android_media_rows_async_inflate,ig_android_stories_gallery_video_segmentation,ig_android_stories_in_feed_preview_notify_fix_universe,ig_android_video_rebind_force_keep_playing_fix,ig_android_direct_business_holdout,ig_android_xposting_upsell_directly_after_sharing_to_story,ig_android_gallery_high_quality_photo_thumbnails,ig_android_interactions_new_comment_like_pos_universe,ig_feed_core_experience_universe,ig_android_friends_sticker,ig_android_business_ix_universe,ig_android_suggested_highlights,ig_android_stories_posting_offline_ui,ig_android_stories_close_friends_rings_remove_green_universe,ig_android_canvas_tilt_to_pan_universe,ig_android_vc_background_call_toast_universe,ig_android_concurrent_cold_start_universe,ig_promote_default_destination_universe,mi_viewpoint_viewability_universe,ig_android_location_page_info_page_upsell,igds_android_listrow_migration_universe,ig_direct_reshare_sharesheet_ranking,ig_android_fb_sync_options_universe,ig_android_drawable_usage_logging_universe,ig_android_recommend_accounts_destination_routing_fix,ig_android_fix_prepare_direct_push,ig_direct_android_larger_media_reshare_style,ig_android_video_feed_universe,ig_android_building_aymf_universe,ig_android_internal_sticker_universe,ig_traffic_routing_universe,ig_android_search_normalization,ig_android_ad_watchmore_entry_point_universe,ig_camera_android_segmentation_enabled_universe,ig_android_igtv_always_show_browse_ui,ig_android_page_claim_deeplink_qe,ig_explore_2018_h2_account_rec_deduplication_android,ig_android_story_accidentally_click_investigation_universe,ig_android_shopping_pdp_hero_carousel,ig_android_clear_inflight_image_request,ig_android_show_su_in_other_users_follow_list,ig_android_stories_infeed_lower_threshold_launch,ig_android_main_feed_video_countdown_timer,instagram_interests_holdout,ig_android_continuous_video_capture,ig_android_category_search_edit_profile,ig_android_contact_invites_nux_universe,ig_android_settings_search_v2_universe,ig_android_video_upload_iframe_interval,ig_business_new_value_prop_universe,ig_android_power_metrics,ig_android_stories_collapse_seen_segments,ig_android_live_follow_from_comments_universe,ig_android_hashtag_discover_tab,ig_android_live_skip_live_encoder_pts_correction,ig_android_reel_zoom_universe,enable_creator_account_conversion_v0_universe,ig_android_test_not_signing_address_book_unlink_endpoint,ig_android_direct_tabbed_media_picker,ig_android_direct_mutation_manager_job_scheduler,ig_ei_option_setting_universe,ig_android_hashtag_related_items_over_logging,ig_android_livewith_liveswap_optimization_universe,ig_android_direct_new_intro_card,ig_camera_android_supported_capabilities_api_universe,ig_android_video_webrtc_textureview,ig_android_share_claim_page_universe,ig_direct_android_mentions_sender,ig_android_whats_app_contact_invite_universe,ig_android_video_scrubber_thumbnail_universe,ig_camera_ar_image_transform_library,ig_android_insights_creation_growth_universe,ig_android_igtv_refresh_tv_guide_interval,ig_android_stories_gif_sticker,ig_android_stories_music_broadcast_receiver,ig_android_fb_profile_integration_fbnc_universe,ig_android_low_data_mode,ig_fb_graph_differentiation_control,ig_android_show_create_content_pages_universe,ig_android_igsystrace_universe,ig_android_new_contact_invites_entry_points_universe,ig_android_ccu_jobscheduler_inner,ig_android_netego_scroll_perf,ig_android_fb_connect_follow_invite_flow,ig_android_invite_list_button_redesign_universe,ig_android_react_native_email_sms_settings_universe,ig_android_igtv_aspect_ratio_limits,ig_hero_player,ig_android_save_auto_sharing_to_fb_option_on_server,ig_android_live_presence_universe,ig_android_whitehat_options_universe,android_cameracore_preview_frame_listener2_ig_universe,ig_android_memory_manager,ig_account_recs_in_chaining,ig_explore_2018_finite_chain_android_universe,ig_android_tagging_video_preview,ig_android_feed_survey_viewpoint,ig_android_hashtag_search_suggestions,ig_android_profile_neue_infra_rollout_universe,ig_android_instacrash_detection,ig_android_interactions_add_search_bar_to_likes_list_universe,ig_android_vc_capture_universe,ig_nametag_local_ocr_universe,ig_branded_content_share_to_facebook,ig_android_direct_segmented_video,ig_android_search_page_v2,ig_android_stories_recently_captured_universe,ig_business_integrity_ipc_universe,ig_android_share_product_universe,ig_fb_graph_differentiation_top_k_fb_coefficients,ig_shopping_viewer_share_action,ig_android_direct_share_story_to_facebook,ig_android_business_attribute_sync,ig_android_video_time_to_live_cache_eviction,ig_android_location_feed_related_business,ig_android_view_and_likes_cta_universe,ig_live_holdout_h2_2018,ig_android_profile_memories_universe,ig_promote_budget_warning_view_universe,ig_android_redirect_to_web_on_oembed_fail_universe,ig_android_optic_new_focus_controller,ig_android_shortcuts,ig_android_search_hashtag_badges,ig_android_navigation_latency_logger,ig_android_direct_composer_avoid_hiding_thread_camera,ig_android_direct_remix_visual_messages,ig_android_custom_story_import_intent,ig_android_biz_new_choose_category,ig_android_view_info_universe,ig_android_camera_upsell_dialog,ig_android_business_ix_self_serve,ig_android_dead_code_detection,ig_android_ad_watchbrowse_universe,ig_android_pbia_proxy_profile_universe,ig_android_qp_kill_switch,ig_android_gap_rule_enforcer_universe,ig_android_direct_delete_or_block_from_message_requests,ig_android_direct_left_aligned_navigation_bar,ig_android_feed_load_more_viewpoint_universe,ig_android_stories_reshare_reply_msg,ig_android_one_tap_sharesheet_fb_extensions,ig_android_stories_feeback_message_composer_entry_point,ig_direct_holdout_h2_2018,ig_camera_android_facetracker_v12_universe,ig_android_camera_ar_effects_low_storage_universe,ig_camera_android_black_feed_sticker_fix_universe,ig_android_direct_media_forwarding,ig_android_camera_attribution_in_direct,ig_android_audience_control,ig_android_stories_cross_sharing_to_fb_holdout_universe,ig_android_enable_main_feed_reel_tray_preloading,ig_android_profile_neue_universe,ig_company_profile_holdout,ig_camera_android_areffect_photo_capture_universe,ig_rti_inapp_notifications_universe,ig_android_vc_join_timeout_universe,ig_android_feed_core_ads_2019_h1_holdout_universe,ig_android_interactions_composer_mention_search_universe,ig_android_igtv_save,ig_android_follower_following_whatsapp_invite_universe,ig_android_claim_location_page,ig_android_story_ads_2019_h1_holdout_universe,ig_android_3pspp,ig_android_cache_timespan_objects,ig_timestamp_public_test,ig_android_histogram_reporter,ig_android_feed_auto_share_to_facebook_dialog,ig_android_arengine_separate_prepare,ig_android_skip_button_content_on_connect_fb_universe,ig_android_igtv_profile_tab,ig_android_show_fb_name_universe,ig_android_interactions_inline_composer_extensions_universe,ig_camera_async_space_validation_for_ar,ig_android_pigeon_sampling,ig_story_camera_reverse_video_experiment,ig_android_live_use_timestamp_normalizer,ig_android_profile_lazy_load_carousel_media,ig_android_stories_question_sticker_music_format,ig_business_profile_18h1_holdout_universe,ig_pacing_overriding_universe,ig_android_direct_allow_multiline_composition,ig_android_interactions_emoji_extension_followup_universe,ig_android_story_ads_direct_cta_universe,ig_android_q3lc_transparency_control_settings,ig_stories_selfie_sticker,ig_android_sso_use_trustedapp_universe,ig_android_ad_increase_story_adpreload_priority_universe,ig_android_interests_netego_dismiss,ig_direct_giphy_gifs_rating,ig_android_shopping_catalogsearch,ig_android_stories_music_awareness_universe,ig_android_qcc_perf,ig_android_stories_reels_tray_media_count_check,ig_android_new_fb_page_selection,ig_android_facebook_crosspost,ig_android_internal_collab_save,ig_video_holdout_h2_2017,ig_android_story_sharing_universe,ig_promote_post_insights_entry_universe,ig_android_direct_thread_store_rewrite,ig_android_qp_clash_management_enabled_v4_universe,ig_branded_content_paid_branded_content,ig_android_large_heap_override,ig_android_live_subscribe_user_level_universe,ig_android_igtv_creation_flow,ig_android_video_call_finish_universe,ig_android_direct_mqtt_send,ig_android_do_not_fetch_follow_requests_on_success,ig_android_remove_push_notifications,ig_android_vc_directapp_integration_universe,ig_android_explore_discover_people_entry_point_universe,ig_android_sonar_prober_universe,ig_android_live_bg_download_face_filter_assets_universe,ig_android_gif_framerate_throttling,ig_android_live_webrtc_livewith_params,ig_android_vc_always_start_connection_on_condition_universe,ig_camera_worldtracking_set_scale_by_arclass,ig_android_direct_inbox_typing_indicator,ig_android_stories_music_lyrics_scrubber,ig_feed_experience,ig_android_direct_new_thread_local_search_fix_universe,ig_android_appstate_logger,ig_promote_insights_video_views_universe,ig_android_dismiss_recent_searches,ig_android_downloadable_igrtc_module,ig_android_fb_link_ui_polish_universe,ig_stories_music_sticker,ig_android_device_capability_framework,ig_scroll_by_two_cards_for_suggested_invite_universe,ig_android_stories_helium_balloon_badging_universe,ig_android_business_remove_unowned_fb_pages,ig_android_stories_combined_asset_search,ig_stories_allow_camera_actions_while_recording,ig_android_analytics_mark_events_as_offscreen,ig_android_optic_feature_testing,ig_android_camera_universe,ig_android_optic_photo_cropping_fixes,ig_camera_regiontracking_use_similarity_tracker_for_scaling,ig_android_refreshable_list_view_check_spring,felix_android_video_quality,ig_android_biz_endpoint_switch,ig_android_direct_continuous_capture,ig_android_comments_direct_reply_to_author,ig_android_vc_webrtc_params,ig_android_claim_or_connect_page_on_xpost,ig_android_anr,ig_android_optic_new_architecture,ig_android_stories_viewer_as_modal_high_end_launch,ig_android_hashtag_follow_chaining_over_logging,ig_new_eof_demarcator_universe,ig_android_push_notifications_settings_redesign_universe,ig_hashtag_display_universe,ig_fbns_push,coupon_price_test_ad4ad_instagram_resurrection_universe,ig_android_live_rendering_looper_universe,ig_android_mqtt_cookie_auth_memcache_universe,ig_android_live_end_redirect_universe,ig_android_direct_mutation_manager_media_2,ig_android_ccu_jobscheduler_outer,ig_smb_ads_holdout_2019_h1_universe,ig_fb_graph_differentiation,ig_android_stories_share_extension_video_segmentation,ig_android_interactions_realtime_typing_indicator_and_live_comments,ig_android_stories_create_flow_favorites_tooltip,ig_android_live_nerd_stats_universe,ig_android_universe_video_production,ig_android_hide_reset_with_fb_universe,ig_android_reactive_feed_like_count,ig_android_stories_music_precapture,ig_android_vc_service_crash_fix_universe,ig_android_shopping_product_overlay,ig_android_direct_double_tap_to_like_hearts,ig_camera_android_api_rewrite_universe,ig_android_growth_fci_team_holdout_universe,ig_android_stories_gallery_recyclerview_kit_universe,ig_android_story_ads_instant_sub_impression_universe,ig_business_signup_biz_id_universe,ig_android_save_all,ig_android_main_feed_fragment_scroll_timing_histogram_uni,ig_android_ttcp_improvements,ig_android_camera_ar_platform_profile_universe,ig_explore_2018_topic_channel_navigation_android_universe,ig_android_live_fault_tolerance_universe,ig_android_stories_viewer_tall_android_cap_media_universe,native_contact_invites_universe,ig_android_dash_script,ig_android_insights_media_hashtag_insight_universe,ig_camera_fast_tti_universe,ig_android_stories_whatsapp_share,ig_android_inappnotification_rootactivity_tweak,ig_android_render_thread_memory_leak_holdout,ig_android_private_highlights_universe,ig_android_rate_limit_feed_video_module,ig_android_one_tap_fbshare,ig_share_to_story_toggle_include_shopping_product,ig_android_direct_speed_cam_univ,ig_payments_billing_address,ig_android_ufiv3_holdout,ig_android_new_camera_design_container_animations_universe,ig_android_livewith_guest_adaptive_camera_universe,ig_android_direct_fix_playing_invalid_visual_message,ig_shopping_viewer_intent_actions,ig_promote_add_payment_navigation_universe,ig_android_optic_disable_post_capture_preview_restart,ig_android_main_feed_refresh_style_universe,ig_android_live_analytics,ig_android_story_ads_performance_universe_1,ig_android_stories_viewer_modal_activity,ig_android_story_ads_performance_universe_3,ig_android_story_ads_performance_universe_4,ig_android_feed_seen_state_with_view_info,ig_android_ads_profile_cta_feed_universe,ig_android_vc_cowatch_universe,ig_android_optic_thread_priorities,ig_android_igtv_chaining,ig_android_live_qa_viewer_v1_universe,ig_android_stories_show_story_not_available_error_msg,ig_android_inline_notifications_recommended_user,ig_shopping_post_insights,ig_android_webrtc_streamid_salt_universe,ig_android_wellbeing_timeinapp_v1_universe,ig_android_profile_cta_v3,ig_android_video_qp_logger_universe,ig_android_cache_video_autoplay_checker,ig_android_live_suggested_live_expansion,ig_android_vc_start_from_direct_inbox_universe,ig_perf_android_holdout,ig_fb_graph_differentiation_only_fb_candidates,ig_android_expired_build_lockout,ig_promote_lotus_universe,ig_android_video_streaming_upload_universe,ig_android_optic_fast_preview_restart_listener,ig_interactions_h1_2019_team_holdout_universe,ig_android_ad_async_ads_universe,ig_camera_android_effect_info_bottom_sheet_universe,ig_android_stories_feedback_badging_universe,ig_android_sorting_on_self_following_universe,ig_android_edit_location_page_info,ig_promote_are_you_sure_universe,ig_android_interactions_feed_label_below_comments_refactor_universe,ig_android_camera_platform_effect_share_universe,ig_stories_engagement_swipe_animation_simple_universe,ig_login_activity,ig_android_direct_quick_replies,ig_android_fbns_optimization_universe,ig_android_stories_alignment_guides_universe,ig_android_rn_ads_manager_universe,ig_explore_2018_post_chaining_account_recs_dedupe_universe,ig_android_click_to_direct_story_reaction_universe,ig_internal_research_settings,ig_android_stories_video_seeking_audio_bug_fix,ig_android_insights_holdout,ig_android_swipe_up_area_universe,ig_android_rendering_controls,ig_android_feed_post_sticker,ig_android_inline_editing_local_prefill,ig_android_hybrid_bitmap_v3_prenougat,ig_android_cronet_stack,ig_android_enable_igrtc_module,ig_android_scroll_audio_priority,ig_android_shopping_product_appeals_universe,ig_android_fb_follow_server_linkage_universe,ig_android_fblocation_universe,ig_android_direct_updated_story_reference_ui,ig_camera_holdout_h1_2018_product,live_with_request_to_join_button_universe,ig_android_music_continuous_capture,ig_android_churned_find_friends_redirect_to_discover_people,ig_android_main_feed_new_posts_indicator_universe,ig_vp9_hd_blacklist,ig_ios_queue_time_qpl_universe,ig_android_split_contacts_list,ig_android_connect_owned_page_universe,ig_android_felix_prefetch_thumbnail_sprite_sheet,ig_android_multi_dex_class_loader_v2,ig_android_watch_and_more_redesign,igtv_feed_previews,ig_android_qp_batch_fetch_caching_enabled_v1_universe,ig_android_profile_edit_phone_universe,ig_android_vc_renderer_type_universe,ig_android_local_2018_h2_holdout,ig_android_purx_native_checkout_universe,ig_android_vc_disable_lock_screen_content_access_universe,ig_android_business_transaction_in_stories_creator,android_cameracore_ard_ig_integration,ig_video_experimental_encoding_consumption_universe,ig_android_iab_autofill,ig_android_location_page_intent_survey,ig_camera_android_segmentation_qe2_universe,ig_android_image_mem_cache_strong_ref_universe,ig_android_business_promote_refresh_fb_access_token_universe,ig_android_stories_samsung_sharing_integration,ig_android_hashtag_header_display,ig_discovery_holdout_2019_h1_universe,ig_android_user_url_deeplink_fbpage_endpoint,ig_android_direct_mutation_manager_handler_thread_universe,ig_branded_content_show_settings_universe,ig_android_ad_holdout_watchandmore_universe,ig_android_direct_thread_green_dot_presence_universe,ig_android_camera_new_post_smile_universe,ig_android_shopping_signup_redesign_universe,ig_android_vc_missed_call_notification_action_reply,allow_publish_page_universe,ig_android_experimental_onetap_dialogs_universe,ig_promote_ppe_v2_universe,android_cameracore_ig_gl_oom_fixes_universe,ig_android_multi_capture_camera,ig_android_fb_family_navigation_badging_user,ig_android_follow_requests_copy_improvements,ig_media_geo_gating,ig_android_comments_notifications_universe,ig_android_render_output_surface_timeout_universe,ig_android_drop_frame_check_paused,ig_direct_raven_sharesheet_ranking,ig_android_realtime_mqtt_logging,ig_family_bridges_holdout_universe,ig_android_rainbow_hashtags,ig_android_ad_watchinstall_universe,ig_android_ad_account_top_followers_universe,ig_android_betamap_universe,ig_android_video_ssim_report_universe,ig_android_cache_network_util,ig_android_leak_detector_upload_universe,ig_android_carousel_prefetch_bumping,ig_fbns_preload_default,ig_android_inline_appeal_show_new_content,ig_fbns_kill_switch,ig_hashtag_following_holdout_universe,ig_android_show_weekly_ci_upsell_limit,ig_android_direct_reel_options_entry_point_2_universe,enable_creator_account_conversion_v0_animation,ig_android_http_service_same_thread,ig_camera_holdout_h1_2018_performance,ig_android_direct_mutation_manager_cancel_fix_universe,ig_music_dash,ig_android_fb_url_universe,ig_android_reel_raven_video_segmented_upload_universe,ig_android_promote_native_migration_universe,ig_camera_android_badge_face_effects_universe,ig_android_hybrid_bitmap_v3_nougat,ig_android_multi_author_story_reshare_universe,ig_android_vc_camera_zoom_universe,ig_android_enable_request_compression_ccu,ig_android_video_controls_universe,ig_android_logging_metric_universe_v2,ig_android_xposting_newly_fbc_people,ig_android_visualcomposer_inapp_notification_universe,ig_android_contact_point_upload_rate_limit_killswitch,ig_android_webrtc_encoder_factory_universe,ig_android_search_impression_logging,ig_android_handle_username_in_media_urls_universe,ig_android_sso_kototoro_app_universe,ig_android_mi_holdout_h1_2019,ig_android_igtv_autoplay_on_prepare,ig_file_based_session_handler_2_universe,ig_branded_content_tagging_upsell,ig_shopping_insights_parity_universe_android,ig_android_live_ama_universe,ig_android_external_gallery_import_affordance,ig_android_updatelistview_on_loadmore,ig_android_optic_new_zoom_controller,ig_android_hide_type_mode_camera_button,ig_android_photos_qpl,ig_android_reel_impresssion_cache_key_qe_universe,ig_android_show_profile_picture_upsell_in_reel_universe,ig_android_live_viewer_tap_to_hide_chrome_universe,ig_discovery_holdout_universe,ig_android_direct_import_google_photos2,ig_android_stories_tray_in_viewer,ig_android_request_verification_badge,ig_android_direct_unlimited_raven_replays_inthreadsession_fix,ig_android_netgo_cta,ig_android_viewpoint_netego_universe,ig_android_stories_separate_overlay_creation,ig_android_iris_improvements,ig_android_biz_conversion_naming_test,ig_android_fci_empty_feed_friend_search,ig_android_hashtag_page_support_places_tab,ig_camera_android_ar_platform_universe,ig_android_stories_viewer_prefetch_improvements,ig_android_optic_camera_warmup,ig_android_place_search_profile_image,ig_android_interactions_in_feed_comment_view_universe,ig_android_fb_sharing_shortcut,ig_android_oreo_hardware_bitmap,ig_android_analytics_diagnostics_universe,ig_android_insights_creative_tutorials_universe,ig_android_vc_universe,ig_android_profile_unified_follow_view,ig_android_collect_os_usage_events_universe,ig_android_shopping_nux_timing_universe,ig_android_fbpage_on_profile_side_tray,ig_android_native_logcat_interceptor,ig_android_direct_thread_content_picker,ig_android_notif_improvement_universe,ig_face_effect_ranking,ig_android_shopping_more_from_business,ig_feed_content_universe,ig_android_hacked_account_reporting,ig_android_disk_usage_logging_universe,ig_android_ad_redesign_iab_universe,ig_android_banyan_migration,ig_android_profile_event_leak_holdout,ig_android_stories_loading_automatic_retry,ig_android_gqls_typing_indicator,ag_family_bridges_2018_h2_holdout,ig_promote_net_promoter_score_universe,ig_android_direct_last_seen_message_indicator,ig_android_biz_conversion_suggest_biz_nux,ig_android_log_mediacodec_info,ig_android_vc_participant_state_callee_universe,ig_camera_android_boomerang_attribution_universe,ig_android_stories_weblink_creation,ig_android_horizontal_swipe_lfd_logging,ig_profile_company_holdout_h2_2018,ig_android_ads_manager_pause_resume_ads_universe,ig_promote_fix_expired_fb_accesstoken_android_universe,ig_android_stories_media_seen_batching_universe,ig_android_interactions_nav_to_permalink_followup_universe,ig_android_live_titles_viewer_side_view_title_universe,ig_android_direct_mark_as_read_notif_action,ig_android_edit_highlight_redesign,ig_android_direct_mutation_manager_backoff_universe,ig_android_interactions_comment_like_for_all_feed_universe,ig_android_mi_skip_analytic_event_pool_universe,ig_android_fbc_upsell_on_dp_first_load,ig_android_audio_ingestion_params,ig_android_video_call_participant_state_caller_universe,ig_fbns_shared,ig_feed_engagement_holdout_2018_h1,ig_camera_android_bg_processor,ig_android_optic_new_features_implementation,ig_android_stories_reel_interactive_tap_target_size,ig_android_video_live_trace_universe,ig_android_igtv_browse_with_pip_v2,ig_android_interactive_listview_during_refresh,ig_android_igtv_feed_banner_universe,ig_android_unfollow_from_main_feed_v2,ig_android_self_story_setting_option_in_menu,ig_android_ad_watchlead_universe,ufi_share,ig_android_live_special_codec_size_list,ig_android_live_qa_broadcaster_v1_universe,ig_android_hide_stories_viewer_list_universe,ig_android_direct_albums,ig_android_business_transaction_in_stories_consumer,ig_android_scroll_stories_tray_to_front_when_stories_ready,ig_android_direct_thread_composer,instagram_android_stories_sticker_tray_redesign,ig_camera_android_superzoom_icon_position_universe,ig_android_business_cross_post_with_biz_id_infra,ig_android_photo_invites,ig_android_reel_tray_item_impression_logging_viewpoint,ig_account_identity_2018_h2_lockdown_phone_global_holdout,ig_android_high_res_gif_stickers,ig_close_friends_v4,ig_fb_cross_posting_sender_side_holdout,ig_android_ads_history_universe,ig_android_comments_composer_newline_universe,ig_rtc_use_dtls_srtp,ig_promote_media_picker_universe,ig_android_live_start_live_button_universe,ig_android_vc_ongoing_call_notification_universe,ig_android_rate_limit_feed_item_viewable_helper,ig_android_bitmap_attribution_check,ig_android_ig_to_fb_sync_universe,ig_android_reel_viewer_data_buffer_size,ig_two_fac_totp_enable,ig_android_vc_missed_call_notification_action_call_back,ig_android_stories_landscape_mode,ig_android_ad_view_ads_native_universe,ig_android_igtv_whitelisted_for_web,ig_android_global_prefetch_scheduler,ig_android_live_thread_delay_for_mute_universe,ig_close_friends_v4_global,ig_android_share_publish_page_universe,ig_android_new_camera_design_universe,ig_direct_max_participants,ig_promote_hide_local_awareness_universe,ig_android_graphql_survey_new_proxy_universe,ig_android_fs_creation_flow_tweaks,ig_android_ad_watchbrowse_cta_universe,ig_android_camera_new_tray_behavior_universe,ig_android_direct_expiring_media_loading_errors,ig_android_show_fbunlink_button_based_on_server_data,ig_android_downloadable_vp8_module,ig_android_igtv_feed_trailer,ig_android_fb_profile_integration_universe,ig_android_profile_private_banner,ig_camera_android_focus_attribution_universe,ig_android_rage_shake_whitelist,ig_android_su_follow_back,ig_android_prefetch_notification_data,ig_android_webrtc_icerestart_on_failure_universe,ig_android_vpvd_impressions_universe,ig_android_payload_based_scheduling,ig_android_grid_cell_count,ig_android_new_highlight_button_text,ig_android_direct_search_bar_redesign,ig_android_hashtag_row_preparer,ig_android_ad_pbia_header_click_universe,ig_android_direct_visual_viewer_ppr_fix,ig_background_prefetch,ig_camera_android_focus_in_post_universe,ig_android_time_spent_dashboard,ig_android_direct_vm_activity_sheet,ig_promote_political_ads_universe,ig_android_stories_auto_retry_reels_media_and_segments,ig_android_recommend_accounts_killswitch,ig_shopping_video_half_sheet,ig_android_ad_iab_qpl_kill_switch_universe,ig_android_interactions_direct_share_comment_universe,ig_android_vc_sounds_universe,ig_camera_android_cache_format_picker_children,ig_android_post_live_expanded_comments_view_universe,ig_android_always_use_server_recents,ig_android_qp_slot_cooldown_enabled_universe,ig_android_asset_picker_improvements,ig_android_direct_activator_cards,ig_android_pending_media_manager_init_fix_universe,ig_android_facebook_global_state_sync_frequency_universe,ig_android_network_trace_migration,ig_android_creation_new_post_title,ig_android_reverse_audio,ig_android_camera_gallery_upload_we_universe,ig_android_direct_inbox_async_diffing_universe,ig_android_live_save_to_camera_roll_limit_by_screen_size_universe,ig_android_profile_phone_autoconfirm_universe,ig_direct_stories_questions,ig_android_optic_surface_texture_cleanup,ig_android_vc_use_timestamp_normalizer,ig_android_post_recs_show_more_button_universe,ig_shopping_checkout_mvp_experiment,ig_android_direct_pending_media,ig_android_scroll_main_feed,ig_android_intialization_chunk_410,ig_android_story_ads_default_long_video_duration,ig_android_interactions_mention_search_presence_dot_universe,ig_android_stories_music_sticker_position,ig_android_direct_character_limit,ig_stories_music_themes,ig_android_nametag_save_experiment_universe,ig_android_media_rows_prepare_10_31,ig_android_fs_new_gallery,ig_android_stories_hide_retry_button_during_loading_launch,ig_android_remove_follow_all_fb_list,ig_android_biz_conversion_editable_profile_review_universe,ig_android_shopping_checkout_mvp,ig_android_local_info_page,ig_android_direct_log_badge_count"

LOGIN_EXPERIMENTS = "ig_android_fci_onboarding_friend_search,ig_android_device_detection_info_upload,ig_android_autosubmit_password_recovery_universe,ig_growth_android_profile_pic_prefill_with_fb_pic_2,ig_account_identity_logged_out_signals_global_holdout_universe,ig_android_background_voice_phone_confirmation_prefilled_phone_number_only,ig_android_login_identifier_fuzzy_match,ig_android_one_tap_aymh_redesign_universe,ig_android_keyboard_detector_fix,ig_android_suma_landing_page,ig_android_direct_main_tab_universe,ig_android_aymh_signal_collecting_kill_switch,ig_android_login_forgot_password_universe,ig_android_smartlock_hints_universe,ig_android_smart_prefill_killswitch,ig_android_account_switch_infra_universe,ig_android_multi_tap_login_new,ig_android_email_one_tap_auto_login_during_reg,ig_android_category_search_in_sign_up,ig_android_report_nux_completed_device,ig_android_reg_login_profile_photo_universe,ig_android_caption_typeahead_fix_on_o_universe,ig_android_nux_add_email_device,ig_android_ci_opt_in_placement,ig_android_remember_password_at_login,ig_type_ahead_recover_account,ig_android_analytics_accessibility_event,ig_sem_resurrection_logging,ig_android_abandoned_reg_flow,ig_android_editable_username_in_reg,ig_android_account_recovery_auto_login,ig_android_sim_info_upload,ig_android_skip_signup_from_one_tap_if_no_fb_sso,ig_android_hide_fb_flow_in_add_account_flow,ig_android_mobile_http_flow_device_universe,ig_account_recovery_via_whatsapp_universe,ig_android_hide_fb_button_when_not_installed_universe,ig_prioritize_user_input_on_switch_to_signup,ig_android_gmail_oauth_in_reg,ig_android_login_safetynet,ig_android_gmail_autocomplete_account_over_one_tap,ig_android_background_voice_phone_confirmation,ig_android_phone_auto_login_during_reg,ig_android_hide_typeahead_for_logged_users,ig_android_hindi,ig_android_reg_modularization_universe,ig_android_bottom_sheet,ig_android_snack_bar_hiding,ig_android_one_tap_fallback_auto_login,ig_android_device_verification_separate_endpoint,ig_account_recovery_with_code_android_universe,ig_android_onboarding_skip_fb_connect,ig_android_phone_reg_redesign_universe,ig_android_universe_noticiation_channels,ig_android_media_cache_cleared_universe,ig_android_account_linking_universe,ig_android_hsite_prefill_new_carrier,ig_android_retry_create_account_universe,ig_android_family_apps_user_values_provider_universe,ig_android_reg_nux_headers_cleanup_universe,ig_android_dialog_email_reg_error_universe,ig_android_ci_fb_reg,ig_android_device_info_foreground_reporting,ig_fb_invite_entry_points,ig_android_device_verification_fb_signup,ig_android_suma_biz_account,ig_android_onetaplogin_optimization,ig_video_debug_overlay,ig_android_ask_for_permissions_on_reg,ig_android_display_full_country_name_in_reg_universe,ig_android_exoplayer_settings,ig_android_persistent_duplicate_notif_checker,ig_android_security_intent_switchoff,ig_android_background_voice_confirmation_block_argentinian_numbers,ig_android_do_not_show_back_button_in_nux_user_list,ig_android_passwordless_auth,ig_android_direct_main_tab_account_switch,ig_android_modularized_dynamic_nux_universe,ig_android_icon_perf2,ig_android_email_suggestions_universe,ig_android_fb_account_linking_sampling_freq_universe,ig_android_prefill_full_name_from_fb,ig_android_access_flow_prefill"

LAUNCHER_CONFIGS = "ig_android_felix_release_players,ig_user_mismatch_soft_error,ig_android_os_version_blocking_config,ig_android_carrier_signals_killswitch,fizz_ig_android,ig_mi_block_expired_events,ig_android_killswitch_perm_direct_ssim,ig_fbns_blocked"

SUPPORTED_CAPABILITIES = [
    {
        "name": "SUPPORTED_SDK_VERSIONS",
        "value": "13.0,14.0,15.0,16.0,17.0,18.0,19.0,20.0,21.0,22.0,23.0,24.0,25.0,26.0,27.0,28.0,29.0,30.0,31.0,32.0,33.0,34.0,35.0,36.0,37.0,38.0,39.0,40.0,41.0,42.0,43.0,44.0,45.0,46.0,47.0,48.0,49.0,50.0,51.0,52.0,53.0",
    },
    {"name": "FACE_TRACKER_VERSION", "value": "12"},
    {"name": "segmentation", "value": "segmentation_enabled"},
    {"name": "WORLD_TRACKER", "value": "WORLD_TRACKER_ENABLED"},
]



from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="instabot",
    version="0.61.0",
    description="Instagram bot scripts for promotion and API python wrapper.",
    long_description=long_description,
    author="Daniil Okhlopkov, Evgeny Kemerov",
    author_email="danokhlopkov@gmail.com, eskemerov@gmail.com",
    license="Apache Software License 2.0",
    url="https://github.com/instagrambot/instabot",
    keywords=["instagram", "bot", "api", "wrapper"],
    install_requires=[
        "tqdm>=4.30.0",
        "requests>=2.21.0",
        "requests-toolbelt>=0.8.0",
        "schedule>=0.6.0",
        "pysocks>=1.6.8",
        "responses>=0.10.5",
        "future>=0.17.1",
        "six>=1.12.0",
        "huepy>=0.9.8.1",
        "pytz>=2019.1",
    ],
    classifiers=[
        # How mature is this project? Common values are
        "Development Status :: 5 - Production/Stable",
        # Indicate who your project is intended for
        "Intended Audience :: Information Technology",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: Apache Software License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
)



from tqdm import tqdm


def like(
    self,
    media_id,
    check_media=True,
    container_module="feed_short_url",
    feed_position=0,
    username=None,
    user_id=None,
    hashtag_name=None,
    hashtag_id=None,
    entity_page_name=None,
    entity_page_id=None,
):

    if not self.reached_limit("likes"):
        if self.blocked_actions["likes"]:
            self.logger.warning("YOUR `LIKE` ACTION IS BLOCKED")
            if self.blocked_actions_protection:
                self.logger.warning(
                    "blocked_actions_protection ACTIVE. Skipping `like` action."
                )
                return False
        self.delay("like")
        if check_media and not self.check_media(media_id):
            return False
        #
        # TODO: commented out simulation "open link in app"
        #
        # if container_module == "feed_short_url":
        #     if "_" in str(media_id):
        #         media_pk = int(media_id.split("_")[0])
        #     else:
        #         media_pk = int(media_id)
        #     link = self.get_link_from_media_id(media_pk)
        #     self.logger.debug("Opening link {}".format(link))
        #     self.api.open_instagram_link(link)
        #     self.logger.debug("Getting media info...")
        #     self.api.media_info(media_id)
        #
        _r = self.api.like(
            media_id,
            container_module=container_module,
            feed_position=feed_position,
            username=username,
            user_id=user_id,
            hashtag_name=hashtag_name,
            hashtag_id=hashtag_id,
            entity_page_name=entity_page_name,
            entity_page_id=entity_page_id,
        )
        if _r == "feedback_required":
            self.logger.error("`Like` action has been BLOCKED...!!!")
            self.blocked_actions["likes"] = True
            return False
        if _r:
            self.logger.info("Liked media %s." % media_id)
            self.total["likes"] += 1
            return True
    else:
        self.logger.info("Out of likes for today.")
    return False


def like_comment(self, comment_id):
    if not self.reached_limit("likes"):
        if self.blocked_actions["likes"]:
            self.logger.warning("YOUR `LIKE` ACTION IS BLOCKED")
            if self.blocked_actions_protection:
                from datetime import timedelta

                next_reset = (self.start_time.date() + timedelta(days=1)).strftime(
                    "%Y-%m-%d %H:%M:%S"
                )
                self.logger.warning(
                    "blocked_actions_protection ACTIVE. Skipping `like` action till, at least, {}.".format(
                        next_reset
                    )
                )
                return False
        self.delay("like")
        _r = self.api.like_comment(comment_id)
        if _r == "feedback_required":
            self.logger.error("`Like` action has been BLOCKED...!!!")
            self.blocked_actions["likes"] = True
            return False
        if _r:
            self.logger.info("Liked comment {}.".format(comment_id))
            self.total["likes"] += 1
            return True
    else:
        self.logger.info("Out of likes for today.")
    return False


def like_media_comments(self, media_id):
    broken_items = []
    media_comments = self.get_media_comments(media_id)
    self.logger.info("Found {} comments".format(len(media_comments)))
    comment_ids = [
        item["pk"]
        for item in media_comments
        if not item.get("has_liked_comment") or not item["has_liked_comment"]
    ]

    if not comment_ids:
        self.logger.info(
            "None comments received: comments not found or comments have been filtered."
        )
        return broken_items

    self.logger.info("Going to like %d comments." % (len(comment_ids)))

    for comment in tqdm(comment_ids):
        if not self.like_comment(comment):
            self.error_delay()
            broken_items = comment_ids[comment_ids.index(comment):]
    self.logger.info("DONE: Liked {count} comments.".format(count=len(comment_ids) - len(broken_items)))
    return broken_items


def like_medias(
    self,
    medias,
    check_media=True,
    container_module="feed_timeline",
    username=None,
    user_id=None,
    hashtag_name=None,
    hashtag_id=None,
    entity_page_name=None,
    entity_page_id=None,
):

    broken_items = []
    if not medias:
        self.logger.info("Nothing to like.")
        return broken_items
    self.logger.info("Going to like %d medias." % (len(medias)))
    feed_position = 0
    for media in tqdm(medias):
        if not self.like(
            media,
            check_media=check_media,
            container_module=container_module,
            feed_position=feed_position,
            username=username,
            user_id=user_id,
            hashtag_name=hashtag_name,
            hashtag_id=hashtag_id,
            entity_page_name=entity_page_name,
            entity_page_id=entity_page_id,
        ):
            self.error_delay()
            broken_items.append(media)
        feed_position += 1
    self.logger.info("DONE: Total liked %d medias." % self.total["likes"])
    return broken_items


def like_timeline(self, amount=None):
    self.logger.info("Liking timeline feed:")
    medias = self.get_timeline_medias()[:amount]
    return self.like_medias(medias, check_media=False)


def like_user(self, user_id, amount=None, filtration=True):
    """ Likes last user_id's medias """
    if filtration:
        if not self.check_user(user_id):
            return False
    self.logger.info("Liking user_%s's feed:" % user_id)
    user_id = self.convert_to_user_id(user_id)
    medias = self.get_user_medias(user_id, filtration=filtration)
    if not medias:
        self.logger.info(
            "None medias received: account is closed or medias have been filtered."
        )
        return False
    return self.like_medias(medias[:amount], filtration)


def like_users(self, user_ids, nlikes=None, filtration=True):
    for user_id in user_ids:
        if self.reached_limit("likes"):
            self.logger.info("Out of likes for today.")
            return
        self.like_user(user_id, amount=nlikes, filtration=filtration)


def like_hashtag(self, hashtag, amount=None):
    """ Likes last medias from hashtag """
    self.logger.info("Going to like media with hashtag #%s." % hashtag)
    medias = self.get_total_hashtag_medias(hashtag, amount)
    if self.api.search_tags(hashtag):
        for tag in self.api.last_json["results"]:
            if tag["name"] == hashtag:
                hashtag_id = tag["id"]
                break
    else:
        self.logger.error("NO INFO FOR HASHTAG: {}".format(hashtag))
        return False
    return self.like_medias(
        medias,
        container_module="feed_contextual_hashtag",
        hashtag_name=hashtag,
        hashtag_id=hashtag_id,
    )


def like_geotag(self, geotag, amount=None):
    # TODO: like medias by geotag
    pass


def like_followers(self, user_id, nlikes=None, nfollows=None):
    self.logger.info("Like followers of: %s." % user_id)
    if self.reached_limit("likes"):
        self.logger.info("Out of likes for today.")
        return
    if not user_id:
        self.logger.info("User not found.")
        return
    follower_ids = self.get_user_followers(user_id, nfollows)
    if not follower_ids:
        self.logger.info("%s not found / closed / has no followers." % user_id)
    else:
        self.like_users(follower_ids[:nfollows], nlikes)


def like_following(self, user_id, nlikes=None, nfollows=None):
    self.logger.info("Like following of: %s." % user_id)
    if self.reached_limit("likes"):
        self.logger.info("Out of likes for today.")
        return
    if not user_id:
        self.logger.info("User not found.")
        return
    following_ids = self.get_user_following(user_id, nfollows)
    if not following_ids:
        self.logger.info("%s not found / closed / has no following." % user_id)
    else:
        self.like_users(following_ids, nlikes)


def like_location_feed(self, place, amount):
    self.logger.info("Searching location: {}".format(place))
    self.api.search_location(place)
    if not self.api.last_json["items"]:
        self.logger.error("{} not found.".format(place))
        return False
    else:
        finded_location = self.api.last_json["items"][0]["location"]["pk"]
        self.api.get_location_feed(finded_location)
        location_feed = self.api.last_json
        if location_feed.get("story"):
            self.logger.info("Liking users from stories...")
            location_to_filter = location_feed["story"]["items"][:amount]
            for i in range(0, len(location_to_filter)):
                user = location_to_filter[i]["user"]["pk"]
                self.like_user(user_id=user, amount=1, filtration=False)
        elif location_feed.get("items"):
            self.logger.info("Liking users from images...")
            max_id = ""
            counter = 0
            while counter < amount:
                location_to_filter = location_feed["items"][:amount]
                medias = self.filter_medias(location_to_filter, filtration=False)
                self.like_medias(medias)
                counter += 1
                if location_feed.get("next_max_id"):
                    max_id = location_feed["next_max_id"]
                else:
                    return False
                self.api.get_location_feed(finded_location, max_id)
                location_feed = self.api.last_json
        else:
            self.logger.error(
                " '{}' does not seem to have pictures. Select a different location.".format(
                    place
                )
            )
            return False
